;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; IDL Widget Interface Procedures. 

; 
; Written winter 2001
; 

PRO Fieldlines_Event, Event

  common setupvariables
  Common maxmin_1
  common time

  widget_control, event.id, get_uvalue = uvalue
  widget_control, event.top, get_uvalue=obj
;
; which button?
;
  case uvalue of
     'Run'            : BEGIN
        obj->Reset_Panel, 1, 1
        obj->Run, uvalue
     END
     'check'            : BEGIN
        obj->Reset_Panel, 2, 1
        obj->Run, uvalue
     END
     'Panel'          : BEGIN
        obj->Reset_Panel, N_Columns, N_Rows
        obj->Run, uvalue
     END
     'Fieldlines_Type': BEGIN
        obj->Set, Type = event.(3)
        obj->Up, event.(3)
     END
     'MPEG': obj->MPEG
     'Fieldlines_Var' : obj->Set, Var  = event.(3)
     'Fieldlines_CT'  : obj->Set, CT   = event.(3)
     'Setup'          : obj->Setup
     else             : obj->Tool_Event, event
  endcase
END

PRO Fieldlines::Set, Type = Type, CT = CT, Var = Var

  if n_elements(Type) NE 0 then Self.Type = Type
  if n_elements(CT)   NE 0 then Self.CT = CT
  if n_elements(Var)  NE 0 then begin
     Self.Var = Var
     Self.VarName = STRUPCASE((*Self.Vars)[Self.Var])+'-lines'
  end
END

PRO Fieldlines::Up, Run_Type
;
; if single file shown, blank out some fields, else unblank these and blank
; out some others
;
  CASE Run_Type OF
    0        : BEGIN
    		 FIELDLINES_EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_EID")
		 WIDGET_CONTROL, FIELDLINES_EID_Ident, SENSITIVE = 0
    		 FIELDLINES_TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_TBI")
		 WIDGET_CONTROL, FIELDLINES_TBI_Ident, SENSITIVE = 0
                 FIELDLINES_CHECK = WIDGET_INFO(Self.Base, FIND_BY_UNAME="check")
                 WIDGET_CONTROL, FIELDLINES_CHECK, SENSITIVE = 1
    		 FIELDLINES_VRML_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_VRML")
		 WIDGET_CONTROL, FIELDLINES_VRML_Ident, SENSITIVE = 1
    		 FIELDLINES_MPEG_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_MPEG")
		 WIDGET_CONTROL, FIELDLINES_MPEG_Ident, SENSITIVE = 0
    		 FIELDLINES_PANEL_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_Panel")
		 WIDGET_CONTROL, FIELDLINES_PANEL_Ident, SENSITIVE = 0
	       END
    1        : BEGIN
    		 FIELDLINES_EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_EID")
		 WIDGET_CONTROL, FIELDLINES_EID_Ident, SENSITIVE = 1
    		 FIELDLINES_TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_TBI")
		 WIDGET_CONTROL, FIELDLINES_TBI_Ident, SENSITIVE = 1
                 FIELDLINES_CHECK = WIDGET_INFO(Self.Base, FIND_BY_UNAME="check")
                 WIDGET_CONTROL, FIELDLINES_CHECK, SENSITIVE = 0
    		 FIELDLINES_VRML_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_VRML")
		 WIDGET_CONTROL, FIELDLINES_VRML_Ident, SENSITIVE = 0
    		 FIELDLINES_MPEG_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_MPEG")
		 WIDGET_CONTROL, FIELDLINES_MPEG_Ident, SENSITIVE = 1
    		 FIELDLINES_PANEL_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_Panel")
		 WIDGET_CONTROL, FIELDLINES_PANEL_Ident, SENSITIVE = 1
    	       END
  END

END

PRO Fieldlines::Setup
  Self.Setup->Realize
END

PRO Fieldlines::Run, uvalue
;
; Make the fieldlines
;
  common setupvariables
  common time
  common startpoints, xa, ya, za

  default, uvalue, " "
  if obj_valid(Self.oMPEG) then self.frame=0
;
; read some input fields
;
  BID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_BID")
  WIDGET_CONTROL, BID_Ident, GET_VALUE=BID
  BID = LONG(BID[0])

  EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_EID")
  WIDGET_CONTROL, EID_Ident, GET_VALUE=EID
  EID = LONG(EID[0])

  TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FIELDLINES_TBI")
  WIDGET_CONTROL, TBI_Ident, GET_VALUE=TBI
  TBI = LONG(TBI[0])

  IF (Self.Type EQ 0) THEN $
    EID = BID
;
; if panel being used, make the object
;
  if uvalue eq 'Panel' then begin
    opanel = obj_new('grpanel', columns=n_columns,rows=n_rows)
  endif
;
; if check button was pressed, make a panel for that
;
  if uvalue eq 'check' then begin
    opanel = obj_new('grpanel', columns=2, rows=1)
  endif

  FOR NID=BID, EID, TBI DO BEGIN
;
; start the loop (if more than one file is to be shown)
;
; figure out where the data is to be read from (own defined variables, 
; own defined structure or from default structure)
;
  if n_elements(veclabels2) ne 0 then begin
    ind = where(veclabels2 eq (*Self.Vars)[Self.Var])
    if ind ne -1 then begin
      data1 = getdata((veccont[0,ind])[0], nid)
      data2 = getdata((veccont[1,ind])[0], nid)
      data3 = getdata((veccont[2,ind])[0], nid)
    endif
  endif else begin
    ind = -1
  endelse
  if n_elements(ownveclabels) ne 0 then begin
    ind2 = where(ownveclabels eq (*Self.Vars)[Self.Var])
    if ind2 ne -1 then begin
      data1 = getdata((ownvecs[0,ind2])[0], nid)
      data2 = getdata((ownvecs[1,ind2])[0], nid)
      data3 = getdata((ownvecs[2,ind2])[0], nid)
    endif
  endif else begin
    ind2 = -1
  endelse
  if ind eq -1 and ind2 eq -1 then begin
      Data1 = GetData((*Self.Vars)[Self.Var]+'1', NID)
      Data2 = GetData((*Self.Vars)[Self.Var]+'2', NID)
      Data3 = GetData((*Self.Vars)[Self.Var]+'3', NID)
  endif
;
; Make the field lines, and rotate them so they become oriented as in setup
;
    S = Size(Data1)
;    print, Self.Setup->GetSx()
    Lines = OBJ_NEW('grFieldlines', Data1, Data2, Data3, Sx=Self.Setup->GetSx(), Sy=Self.Setup->GetSy(), Sz=Self.Setup->GetSz(), nSteps=Self.Setup->GetnSteps(), steplen=Self.Setup->Getsteplen(), DimScaleX=DimScalex, DimScaleY=DimScaley, DimScaleZ=DimScalez)

    Lines->Rotate, [1,0,0], Self.Setup->GetAx1()
    Lines->Rotate, [0,1,0], Self.Setup->GetAx2()
    Lines->Rotate, [0,0,1], Self.Setup->GetAx3()

    if (NID eq BID) then begin
      z = Self.Setup->GetZoom()

      Lines->GetProperty, Transform = T
;
; Find the size of the object, to resize the view as it should be
;
      cor = fltarr(2,8)
      if  (z ne 0) then begin
        cor[*,0] = CVT_TO_2D(DimScaleX[0],DimScaleY[0],DimScaleZ[0],T)
        cor[*,1] = CVT_TO_2D(DimScaleX[0],DimScaleY[0],DimScaleZ[N_Elements(DimScaleZ)-1],T)
        cor[*,2] = CVT_TO_2D(DimScaleX[0],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[0],T)
        cor[*,3] = CVT_TO_2D(DimScaleX[0],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[N_Elements(DimScaleZ)-1],T)
        cor[*,4] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[0],DimScaleZ[0],T)
        cor[*,5] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[0],DimScaleZ[N_Elements(DimScaleZ)-1],T)
        cor[*,6] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[0],T)
        cor[*,7] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[N_Elements(DimScaleZ)-1],T)
      end else begin
        x1min = Lines->GetX1Min()
        x1max = Lines->GetX1Max()
        x2min = Lines->GetX2Min()
        x2max = Lines->GetX2Max()
        x3min = Lines->GetX3Min()
        x3max = Lines->GetX3Max()
        cor[*,0] = CVT_TO_2D(x1min,x2min,x3min,T)
        cor[*,1] = CVT_TO_2D(x1min,x2min,x3max,T)
        cor[*,2] = CVT_TO_2D(x1min,x2max,x3min,T)
        cor[*,3] = CVT_TO_2D(x1min,x2max,x3max,T)
        cor[*,4] = CVT_TO_2D(x1max,x2min,x3min,T)
        cor[*,5] = CVT_TO_2D(x1max,x2min,x3max,T)
        cor[*,6] = CVT_TO_2D(x1max,x2max,x3min,T)
        cor[*,7] = CVT_TO_2D(x1max,x2max,x3max,T)
      end
    end
;
; Find the corners and the size from the above calculations
;
    if  (z ne 0) then begin
      c1 = Min(cor(0,*))
      c2 = Min(cor(1,*))/z
      l1 = (Max(cor(0,*))-Min(cor(0,*)))/z
      l2 = (Max(cor(1,*))-Min(cor(1,*)))/z
    end else begin
      c1 = Min(cor(0,*))
      c2 = Min(cor(1,*))
      l1 = (Max(cor(0,*))-Min(cor(0,*)))
      l2 = (Max(cor(1,*))-Min(cor(1,*)))
    end
    maxx=(max(dimscalex)-min(dimscalex))/(max(dimscaley)-min(dimscaley))
;
; Not used
;
;;    Lines->getproperty, fieldline=fieldline, Sy=Sy, Sz=Sx, /indicies
;;    N = UINT(N_ELEMENTS(Fieldline)/3.)
;;    S1 = TOTAL([0, REFORM(SQRT(Fieldline[0,1:N-1]-Fieldline[0,0:N-2])^2)])
;;    S2 = TOTAL([0, REFORM(SQRT((Fieldline[1,1:N-1]-Fieldline[1,0:N-2])^2))])
;;    S3 = TOTAL([0, REFORM(SQRT((Fieldline[2,1:N-1]-Fieldline[2,0:N-2])^2))])
;
; Make the titles, model and view 
;
    if (uvalue eq 'Panel') and (n_rows gt 1) then begin
      ofont = obj_new('idlgrfont', 'Helvetica', size=14/n_rows[0])
      if (float(n_rows[0]) eq 1) and (float(n_columns[0]) ne 1) then $
       ofont = obj_new('idlgrfont', 'Helvetica', size=14/2.) 
    endif else $
      ofont = obj_new('idlgrfont', 'Helvetica', size=14)

    oTitle = OBJ_NEW('IDLgrText', font=ofont, strings=Self.Varname,locations=[l1*.5+c1,l2*.98+c2],/onglass, VERTICAL_ALIGNMENT=1.0, alignment=0.5)

    oTitle2 = OBJ_NEW('IDLgrText', font=ofont, strings='Time = '+string(dt*NID, format='(i3)'),locations=[c1+0.98*l1,c2+0.02*l2],/onglass, VERTICAL_ALIGNMENT=0.0, alignment=1.0)

    oModel = OBJ_NEW('IDLgrModel')
    oView = OBJ_NEW('IDLgrView', VIEWPLANE_RECT=[c1,c2,l1,l2], ZClip = [150, -150], Eye = S[1] + 1)

    oModel->Add, oTitle
    oModel->Add, oTitle2
    oView->Add, Lines
    oView->Add, oModel

    if uvalue eq 'check' then begin
;
; If check button pressed, find the fieldlines again by interpolating
; outside to in
;
      Linesb = OBJ_NEW('grFieldlines', Data1, Data2, Data3, Sx=Self.Setup->GetSx(), Sy=Self.Setup->GetSy(), Sz=Self.Setup->GetSz(), nSteps=Self.Setup->GetnSteps(), steplen=Self.Setup->Getsteplen(), DimScaleX=DimScaleX, DimScaleY=DimScaleY, DimScaleZ=DimScaleZ, /backwards)
      Linesb->Rotate, [1,0,0], Self.Setup->GetAx1()
      Linesb->Rotate, [0,1,0], Self.Setup->GetAx2()
      Linesb->Rotate, [0,0,1], Self.Setup->GetAx3()
      oTitleb = OBJ_NEW('IDLgrText', font=ofont, strings='Interpolating backwards',locations=[l1*.5+c1,l2*.98+c2],/onglass, VERTICAL_ALIGNMENT=1.0, alignment=0.5)
 
      oTitle2b = OBJ_NEW('IDLgrText', font=ofont, strings='Time = '+string(dt*NID, format='(i3)'),locations=[c1+0.98*l1,c2+0.02*l2],/onglass, VERTICAL_ALIGNMENT=0.0, alignment=1.0)

      oModelb = OBJ_NEW('IDLgrModel')
      oViewb = OBJ_NEW('IDLgrView', VIEWPLANE_RECT=[c1,c2,l1,l2], ZClip = [150, -150], Eye = S[1] + 1)

      oModelb->Add, oTitleb
      oModelb->Add, oTitle2b
      oViewb->Add, Linesb
      oViewb->Add, oModelb

    endif
    if uvalue eq 'check' then begin
;
; if check, then add the both views to the panel
;
      opanel->add, oview
      opanel->add, oviewb
    endif else $
;
; if using panel, add view to the panel
; 
    if uvalue eq 'Panel' then begin
     opanel -> add, oview
    endif else begin
;
; if not, just draw the view
;
     Self->resize,xstretch=1.0
     Self->Draw, oView
    endelse
  ENDFOR
;
; if panel is used, draw the panel. Notice that this is outside the loop.
;
    if (uvalue eq 'Panel') or (uvalue eq 'check') then begin
      if uvalue eq 'Panel' then self->resize, xstretch=n_columns/float(n_rows)
      if uvalue eq 'check' then self->resize, xstretch=2
      self->draw, opanel
    endif
;
; if checking, print some info to the screen
;
    if uvalue eq 'check' then begin
      diffx2=(xa-Self.Setup->GetSx())^2
      diffy2=(ya-Self.Setup->GetSy())^2
      diffz2=(za-Self.Setup->GetSz())^2
      print, "The difference on the starting points and the new endpoints are: "
      print, sqrt(diffx2+diffy2+diffz2)
      print, "The maximum difference is: "
      print, max(sqrt(diffx2+diffy2+diffz2))
      print, "The mean difference is: "
      print, total(sqrt(diffx2+diffy2+diffz2))/n_elements(diffx2)
    endif
    print, "Fieldlines Done"
END

FUNCTION fieldlines::init

common setupvariables
common new_variables

  status = Self->Tool::Init('Fieldlines')

  Self.Setup = obj_new('fieldlinessetup')
  
  TYPE_LIST = FSC_Droplist(Self.InBase, Value=['Single','Multiple'], Index=0, UValue='Fieldlines_Type', TITLE = 'File(s) :   ')

  Self.Vars = ptr_new(1)
;
; figure out which vectors are available
;
  if n_elements(ownveclabels) gt 0 then begin
   if n_elements(veclabels2) ne 0 then begin
    *Self.Vars = [ownveclabels(where(ownveclabels ne '0')), veclabels2(where(veclabels2 ne '0'))]
   endif else begin
    if (where(ownveclabels ne '0'))[0] ne -1 then $
     *Self.Vars = [ownveclabels(where(ownveclabels ne '0'))]
   endelse
  endif else begin
   *Self.Vars = ['b','v']
  endelse
   if n_elements(veclabels2) ne 0 then begin 
     if (where(veclabels2 ne '0' and veclabels2 ne ''))[0] ne -1 then begin
       if n_elements(*self.vars) ne 0 then begin
         *Self.Vars = ['b','v',veclabels2(where(veclabels2 ne '0' and veclabels2 ne ''))]
       endif else begin
         *Self.Vars = [veclabels2(where(veclabels2 ne '0' and veclabels2 ne ''))]
       endelse
     endif
   endif

  FIELDLINES_Var_LIST = FSC_Droplist(Self.InBase, Value=*Self.Vars, Index=0, UVALUE='Fieldlines_Var', TITLE = 'Variable :  ')

  FileTitle = Widget_Label(Self.InBase, /ALIGN_LEFT, VALUE='File ID(s)')
;
; the input fields
; 
  BID_CW = CW_FIELD(Self.InBase, UNAME = 'FIELDLINES_BID', TITLE = 'First    ', VALUE = 0)
  EID_CW = CW_FIELD(Self.InBase, UNAME = 'FIELDLINES_EID', TITLE = 'Last     ', VALUE = 60)
  TBI_CW = CW_FIELD(Self.InBase, UNAME = 'FIELDLINES_TBI', TITLE = 'Increment', VALUE = 1)
;
; the buttons
;
  FIELDLINES_RUN_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE ='Run', VALUE='Run')

  FIELDLINES_TEST_BUTTON = widget_button(self.inbase, uname = 'check', uvalue = 'check', value = 'Check')

  FIELDLINES_PRINT_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='PS', VALUE='Save')

  FIELDLINES_VRML_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='VRML', VALUE='VRML', UNAME='FIELDLINES_VRML')

  FIELDLINES_PANEL_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='Panel', VALUE='Panel',UNAME='FIELDLINES_Panel')

  FIELDLINES_MPEG_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='MPEG', VALUE='MPEG',UNAME='FIELDLINES_MPEG')
  
  SETUP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='Setup',VALUE='Setup')

  FIELDLINES_HELP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='Help',VALUE='Help')

  FIELDLINES_CLOSE_BUTTON = WIDGET_BUTTON(Self.InBase, UValue='Close', Value='Close')

  Self->Up, 0

  Self->Resize

  Self.Helpfile = "help/HELP_FIELDLINES"
  Self.Varname = STRUPCASE((*Self.Vars)[Self.Var])+'-lines'

  return, 1
end

PRO fieldlines__define
  void = {fieldlines, Type: 0, Var: 0, Vars: ptr_new(1), CT: 0, $
          setup: obj_new(), $
          INHERITS tool}
END
