;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; IDL Widget Interface Procedures. 
;
; 
; Written winter 2001
; 

PRO Fieldlines_setup_Event, Event

  common setupvariables

  widget_control, event.id, get_uvalue = uvalue
  widget_control, event.top, get_uvalue=obj

  case uvalue of
     'Run'            : BEGIN
        obj->Reset_Panel, 1, 1
        obj->Run
     END
     'Panel'          : BEGIN
        obj->Reset_Panel, N_Columns, N_Rows
        obj->Run
     END
     'StartingpointsType': BEGIN
        obj->Set, Type = event.(3)
        obj->Up, event.(3)
     END
     'Fieldlines_Var' : obj->Set, Var  = event.(3)
     'Fieldlines_CT'  : obj->Set, CT   = event.(3)
     'Save': obj->Save
     'Ax1_Slider' : begin
        obj->Set, Ax1 = event.(3)
        obj->Render
     end
     'Ax2_Slider' : begin
        obj->Set, Ax2 = event.(3)
        obj->Render
     end
     'Ax3_Slider' : begin
        obj->Set, Ax3 = event.(3)
        obj->Render
     end
     else             : obj->Tool_Event, event
  endcase
END

FUNCTION Fieldlinessetup::GetnSteps

  return, Self.nSteps
END

FUNCTION Fieldlinessetup::GetSteplen

  return, Self.steplen
END

FUNCTION Fieldlinessetup::GetSx

  return, *Self.Sx
END

FUNCTION Fieldlinessetup::GetSy

  return, *Self.Sy
END

FUNCTION Fieldlinessetup::GetSz

  return, *Self.Sz
END

FUNCTION Fieldlinessetup::GetZoom

  return, Self.Zoom
END

FUNCTION Fieldlinessetup::GetAx1

  return, Self.Ax1
END

FUNCTION Fieldlinessetup::GetAx2

  return, Self.Ax2
END

FUNCTION Fieldlinessetup::GetAx3

  return, - Self.Ax3
END

PRO Fieldlinessetup::Set, Type = Type, CT = CT, Var = Var, Ax1 = Ax1, Ax2 = Ax2, Ax3 = Ax3

  if n_elements(Type) NE 0 then Self.Type = Type
  if n_elements(CT)   NE 0 then Self.CT  = CT
  if n_elements(Var)  NE 0 then Self.Var = Var
  if n_elements(Ax1)  NE 0 then Self.Ax1 = Ax1
  if n_elements(Ax2)  NE 0 then Self.Ax2 = Ax2
  if n_elements(Ax3)  NE 0 then Self.Ax3 = Ax3
END

PRO Fieldlinessetup::Up, Run_Type


END

PRO fieldlinessetup::CalcStartingPoints

  common setupvariables

  CASE Self.Type OF
     0: Begin
        if self.nlines gt 1 then begin
;
; we need an even number of fieldlines if rin is nonzero:
;
         if (self.rin gt 0) and (self.nlines mod 2 ne 0) then self.nlines=self.nlines+1
; if rin is zero, we need an uneven number of fieldlines. However, two fieldlines will be
; drawn on top of each other, so we make it an even number:
         if (self.rin eq 0) and (self.nlines mod 2 ne 0) then self.nlines=self.nlines+1
         if self.nlines gt 2 then r2=findgen(self.nlines/2)*(self.rout-self.rin)/(self.nlines/2-1)+self.rin
         if self.nlines eq 2 then r2=(self.rout-self.rin)/2+self.rin
         r1=-r2
         r=[r1,r2]
        endif else $
         r = (self.rout-self.rin)/2.
        Phi = !pi/Self.NBase*findgen(Self.NBase)
        sy = r#cos(Phi)+self.centerx
        sz = r#sin(Phi)+self.centery
;        Sy = r#cos(Phi)+(max(dimscaley)+min(dimscaley))/2.
;        Sz = r#sin(Phi)+(max(dimscalez)+min(dimscalez))/2.
     END
     1: BEGIN
        if (Self.NBase LE 1) then Self.NBase = 2
        r = (Self.Rout-Self.Rin)/(Self.NBase-1.)*(findgen(Self.NBase)-Self.NBase/2.)+Self.Rin
        Phi = 2*!pi/Self.NLines*indgen(Self.NLines)
        sy = r#cos(Phi)+self.centerx
        sz = r#sin(Phi)+self.centery
;        Sy = r#cos(Phi)+(max(dimscaley)+min(dimscaley))/2.
;        Sz = r#sin(Phi)+(max(dimscalez)+min(dimscalez))/2.      
     END
     2: BEGIN
        r = (Self.Rout-Self.Rin)/(Self.NLines*Self.NBase-1)*(indgen(Self.NLines*Self.NBase))+Self.Rin
        Phi = 2.*!pi/Self.NLines*findgen(Self.NBase*Self.NLines)
        sy = r#cos(Phi)+self.centerx
        sz = r#sin(Phi)+self.centery
;        Sy = r*cos(Phi)+(max(dimscaley)+min(dimscaley))/2.
;        Sz = r*sin(Phi)+(max(dimscalez)+min(dimscalez))/2.
      END
  END

  Sx = fltarr(Self.NLines*Self.NBase)
  Sx[*] = DimScaleX[0] + 0.01

  *Self.Sx = Sx
  *Self.Sy = Sy
  *Self.Sz = Sz
END

PRO Fieldlinessetup::Run

  common setupvariables

  NBaseLines_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="NBaseLines")
  WIDGET_CONTROL, NBaseLines_Ident, GET_VALUE=NBaseLines
  Self.NBase = LONG(NBaseLines[0])

  NLines_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="NLines")
  WIDGET_CONTROL, NLines_Ident, GET_VALUE=NLines
  Self.NLines = LONG(NLines[0])

  Rin_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="Rin")
  WIDGET_CONTROL, Rin_Ident, GET_VALUE=Rin
  Self.Rin = FLOAT(Rin[0])

  Rout_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="Rout")
  WIDGET_CONTROL, Rout_Ident, GET_VALUE=Rout
  Self.Rout = FLOAT(Rout[0])

  NSteps_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="NSteps")
  WIDGET_CONTROL, nSteps_Ident, GET_VALUE=nSteps
  Self.nSteps = LONG(nSteps[0])

  steplen_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="steplength")
  WIDGET_CONTROL, steplen_Ident, GET_VALUE=steplen
  Self.steplen = FLOAT(steplen[0])

;  Zoom_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="Zoom")
;  WIDGET_CONTROL, Zoom_Ident, GET_VALUE=Zoom
;  Self.Zoom = LONG(Zoom[0])

   widget_control, widget_info(self.base, find_by_uname="centerx"), get_value=centerx
   self.centerx = centerx[0]

   widget_control, widget_info(self.base, find_by_uname="centery"), get_value=centery
   self.centery = centery[0]

  Self->CalcStartingPoints

  Self->Render, /NOAngles
  Wait, 1.5
  Self->Render

end

PRO Fieldlinessetup::Render, NOAngles=NOAngles

  common setupvariables

  sizex=(max(dimscalex)-min(dimscalex))*0.02
  sizey=(max(dimscaley)-min(dimscaley))*0.02
  sizez=(max(dimscalez)-min(dimscalez))*0.02
  oSym = OBJ_NEW('IDLgrSymbol', size=[sizex, sizey, sizez])
  oJetAxis = OBJ_NEW('IDLgrPolyline',[0.5,0.5],[0,0],[0,DimScaleX[n_elements(DimScaleX) - 1]],color=[255,0,0])
  oPlot = OBJ_NEW('IDLgrPlot', *Self.Sz, *Self.Sy, LINESTYLE=6, SYMBOL=oSym)
;  oPlot = OBJ_NEW('IDLgrPlot', *Self.Sy, *Self.Sz, LINESTYLE=6, SYMBOL=oSym)
  oModel = OBJ_NEW('IDLgrModel')
  oView = OBJ_NEW('IDLgrView', ViewPlane_Rect=[DimScaleZ[0],DimScaleY[0],DimScaleZ[n_elements(DimScaleZ)-1]-DimScaleZ[0],DimScaleY[n_elements(DimScaleY)-1]-DimScaleY[0]], zclip=[DimScaleZ[n_elements(DimScaleZ)-1],-DimScaleZ[n_elements(DimScaleZ)-1]], EYE=DimScaleZ[n_elements(DimScaleZ)-1]+1)
;  oView = OBJ_NEW('IDLgrView', ViewPlane_Rect=[DimScaleY[0],DimScaleZ[0],DimScaleY[n_elements(DimScaleY)-1]-DimScaleY[0],DimScaleZ[n_elements(DimScaleZ)-1]-DimScaleZ[0]], zclip=[DimScaleZ[n_elements(DimScaleZ)-1],-DimScaleZ[n_elements(DimScaleZ)-1]], EYE=DimScaleZ[n_elements(DimScaleZ)-1]+1)
  oModel->Add, oPlot
  oModel->Add, oJetAxis

  if not(keyword_set(NOAngles)) then begin
     oModel->Rotate, [0,0,1], Self.Ax1
     oModel->Rotate, [0,1,0], Self.Ax2
     oModel->Rotate, [1,0,0], Self.Ax3
;     omodel->rotate, [0,0,1], 90
     oModel->Rotate, [0,1,0], 90
     omodel->getproperty, transform = t
     cor = fltarr(2,8)
;        cor[*,0] = CVT_TO_2D(DimScaleX[0],DimScaleY[0],DimScaleZ[0],T)
;        cor[*,1] = CVT_TO_2D(DimScaleX[0],DimScaleY[0],DimScaleZ[N_Elements(DimScaleZ)-1],T)
;        cor[*,2] = CVT_TO_2D(DimScaleX[0],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[0],T)
;        cor[*,3] = CVT_TO_2D(DimScaleX[0],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[N_Elements(DimScaleZ)-1],T)
;        cor[*,4] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[0],DimScaleZ[0],T)
;        cor[*,5] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[0],DimScaleZ[N_Elements(DimScaleZ)-1],T)
;        cor[*,6] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[0],T)
;        cor[*,7] = CVT_TO_2D(DimScaleX[N_Elements(DimScaleX)-1],DimScaleY[N_Elements(DimScaleY)-1],DimScaleZ[N_Elements(DimScaleZ)-1],T)
        ojetaxis->getproperty, data=data
        x1min = min(*self.sz and data[2,*])
        x1max = max(*self.sz and data[2,*])
        x2min = min(*self.sx and data[0,*])
        x2max = max(*self.sx and data[0,*])
        x3min = min(*self.sy and data[1,*])
        x3max = max(*self.sy and data[1,*])

        cor[*,0] = CVT_TO_2D(x1min,x2min,x3min,T)
        cor[*,1] = CVT_TO_2D(x1min,x2min,x3max,T)
        cor[*,2] = CVT_TO_2D(x1min,x2max,x3min,T)
        cor[*,3] = CVT_TO_2D(x1min,x2max,x3max,T)
        cor[*,4] = CVT_TO_2D(x1max,x2min,x3min,T)
        cor[*,5] = CVT_TO_2D(x1max,x2min,x3max,T)
        cor[*,6] = CVT_TO_2D(x1max,x2max,x3min,T)
        cor[*,7] = CVT_TO_2D(x1max,x2max,x3max,T)

      c1 = Min(cor(0,*))   
      c2 = Min(cor(1,*)) 
      l1 = (Max(cor(0,*))-Min(cor(0,*)))*2
      l2 = (Max(cor(1,*))-Min(cor(1,*)))*2
      c1=-self.rout*1.1
      c2=c1
      l1=self.rout*2*1.1
      l2=self.rout*2*1.1
      oview->setproperty, viewplane_rect=[c1,c2,l1,l2]
  end

  oView->Add, oModel
  Self->Draw, oView, /nosave
END

PRO FieldlinesSetup::Realize

  Self->WidInit
  
  StartTitle = Widget_Label(Self.InBase, /ALIGN_LEFT, VALUE='Starting Points')

  TYPE_LIST = FSC_Droplist(Self.InBase, Value=['Lines','Circles','Spirals'], Index=Self.Type, UValue='StartingpointsType', TITLE = 'Arrangement')

  NBase_CW = CW_FIELD(Self.InBase, UNAME = 'NBaseLines', TITLE = 'NBase       ', VALUE = Self.NBase)

  NLines_CW = CW_FIELD(Self.InBase, UNAME = 'NLines', TITLE = 'NLines      ', VALUE = Self.NLines)

  rin_CW = CW_FIELD(Self.InBase, UNAME = 'Rin', TITLE = 'Inner R     ', VALUE = Self.Rin)

  rout_CW = CW_FIELD(Self.InBase, UNAME = 'Rout', TITLE = 'Outer R     ', VALUE = Self.Rout)

  nsteps_CW = CW_FIELD(Self.InBase, UNAME = 'NSteps', TITLE = '# Steps     ', VALUE = Self.nSteps)

  steplen_CW = CW_FIELD(Self.InBase, UNAME = 'steplength', TITLE = 'Step length ', VALUE = Self.steplen)

;  zoom_CW = CW_FIELD(Self.InBase, UNAME = 'Zoom', TITLE = 'Zoom   ', VALUE = Self.Zoom)

  centerx_cw= cw_field(self.inbase, uname = 'centerx', title = 'X Center    ', value = self.centerx)

  centery_cw= cw_field(self.inbase, uname = 'centery', title = 'Y Center    ', value = self.centery)

  rotationlabel = widget_label(self.Inbase, value='Rotation around axis: ', /align_left)

  Ax1_slider = WIDGET_SLIDER(Self.InBase, UVALUE = 'Ax1_Slider', TITLE = 'x1', Minimum=0, Maximum=90, Value=Self.Ax1)

  Ax2_slider = WIDGET_SLIDER(Self.InBase, UVALUE = 'Ax2_Slider', TITLE = 'x2', Minimum=0, Maximum=90, Value=Self.Ax2)

  Ax3_slider = WIDGET_SLIDER(Self.InBase, UVALUE = 'Ax3_Slider', TITLE = 'x3', Minimum=0, Maximum=90, Value=Self.Ax3)

  FSetup_Use_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE ='Run', VALUE='Use')

  FSetup_Save_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE ='Save', VALUE='Save')

  FIELDLINES_HELP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='Help',VALUE='Help')

  FIELDLINESsetup_Close_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='Close',VALUE='Close')

  Self->Up, 0

  Self->Resize
 
END

PRO fieldlinessetup::save

  home = GETENV('HOME')
  if home EQ '' THEN $
    filename = '.jetget_fieldlines' $
  ELSE $
    filename = home + '/.jetget_fieldlines'

  openw, ConfFile, filename, /get_lun
  printf, ConfFile, Self.type
  printf, ConfFile, Self.nSteps
  printf, ConfFile, Self.Rin
  printf, ConfFile, Self.Rout
  printf, ConfFile, Self.NBase
  printf, ConfFile, Self.NLines
  printf, ConfFile, Self.Steplen
  printf, ConfFile, Self.Ax1
  printf, ConfFile, Self.Ax2
  printf, ConfFile, Self.Ax3
  printf, ConfFile, Self.Zoom
  printf, ConfFile, Self.centerx
  printf, ConfFile, Self.centery

  close, ConfFile
  free_lun, ConfFile
END

FUNCTION fieldlinessetup::init, _Extra=e

  status = Self->Tool::Init('Fieldlines_Setup', /NoRealize, _Extra=e)

  Self.Sx = ptr_new(1)
  Self.Sy = ptr_new(1)
  Self.Sz = ptr_new(1)

  home = GETENV('HOME')
  if home EQ '' THEN $
    filename = '.jetget_fieldlines' $
  ELSE $
    filename = home + '/.jetget_fieldlines'

  if fileExists(filename) then begin
    openr, ConfFile, filename, /get_lun
    readf, ConfFile, Type
    Self.Type = Type
    readf, ConfFile, nSteps
    Self.nSteps = nSteps
    readf, ConfFile, Rin
    Self.Rin = Rin
    readf, ConfFile, Rout
    Self.Rout = Rout
    readf, ConfFile, NBase
    Self.NBase = NBase
    readf, ConfFile, NLines
    Self.NLines = NLines
    readf, ConfFile, steplen
    self.steplen = steplen
    readf, ConfFile, Ax1
    Self.Ax1 = Ax1
    readf, ConfFile, Ax2
    Self.Ax2 = Ax2
    readf, ConfFile, Ax3
    Self.Ax3 = Ax3
    readf, ConfFile, Zoom
    Self.Zoom = Zoom
    readf, conffile, centerx
    Self.centerx = centerx
    readf, conffile, centery
    Self.centery = centery
    close, ConfFile
    free_lun, ConfFile
 end else begin
    Self.nSteps = 1001
    Self.Rin = 1.
    Self.Rout = 15.
    Self.NBase = 1
    Self.NLines = 10
    Self.Steplen = 0.005
    Self.Type = 0
    Self.Ax1 = 0
    Self.Ax2 = 0
    Self.Ax3 = 0
    Self.Zoom = 0.
    self.centerx = 0.
    self.centery = 0.
 end

 Self.Helpfile = "help/HELP_FIELDLINESSETUP"
 Self->CalcStartingPoints

 return, 1
end

PRO fieldlinesSetup__define
  void = {FieldlinesSetup, Type: 0, steplen: 0.005, nSteps: 1001, Sx: ptr_new(), Sy: ptr_new(), Sz: ptr_new(), Rin: 0., Rout: 0., NBase: 0, NLines: 0, Ax1: 0, Ax2: 0, Ax3: 0, Zoom: 0., centerx: 0., centery:0., $
          INHERITS tool}
END
