;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; Program for performing fourier convolution 
; as described in Ouyed, Pudritz and Clarke 2001
;
;	R. Ouyed (2001)
;
; To be accessed from JETGET main widget.
;

PRO fourier_Event, event

  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 

  widget_control,event.top,get_uvalue=obj
  widget_control,event.id,get_uvalue=mystic
;
; what button is pressed
;
  case mystic of 
;
; clear or unclear some fields depending on whether one or several files
; should be used
;
    'fourier_filelist' : begin 
                        obj->Set, filelist = event.(3)
                        obj->up, event.(3)
;                        if event.(3) eq 0 then begin
;                         cclear1=widget_info(event.top, find_by_uname="eid")
;                         widget_control, cclear1, sensitive=0
;                         cclear2=widget_info(event.top, find_by_uname="sid")
;                         widget_control, cclear2, sensitive=0
;                         cclear3=widget_info(event.top, find_by_uname="r0")
;                         widget_control, cclear3, sensitive=1
;                         cclear4=widget_info(event.top, find_by_uname="dr0")
;                         widget_control, cclear4, sensitive=1
;                        endif
;                        if event.(3) eq 1 then begin
;                         uncclear1=widget_info(event.top, find_by_uname="eid")
;                         widget_control, uncclear1, sensitive=1
;                         uncclear2=widget_info(event.top, find_by_uname="sid")
;                         widget_control, uncclear2, sensitive=1
;                         uncclear3=widget_info(event.top, find_by_uname="r0")
;                         widget_control, uncclear3, sensitive=0
;                         uncclear4=widget_info(event.top, find_by_uname="dr0")
;                         widget_control, uncclear4, sensitive=1
;                       endif
                      end

    'selectfilter': a=obj_new('fourierselectfilter')
;    'XPalette' : XPalette
    'showfilter': obj->showfilter
    'kvsm': obj->kvsm
    'amplitudep': obj->amplitudep
    'ps': obj->tool_event,event
    'Help': obj->Tool_Event, event
  endcase

END

PRO fourier::up, i
                        if i eq 0 then begin
                         cclear1=widget_info(self.base, find_by_uname="eid")
                         widget_control, cclear1, sensitive=0
                         cclear2=widget_info(self.base, find_by_uname="sid")
                         widget_control, cclear2, sensitive=0
                         cclear3=widget_info(self.base, find_by_uname="r0")
                         widget_control, cclear3, sensitive=1
                         cclear4=widget_info(self.base, find_by_uname="dr0")
                         widget_control, cclear4, sensitive=1
                        endif
                        if i eq 1 then begin
                         uncclear1=widget_info(self.base, find_by_uname="eid")
                         widget_control, uncclear1, sensitive=1
                         uncclear2=widget_info(self.base, find_by_uname="sid")
                         widget_control, uncclear2, sensitive=1
                         uncclear3=widget_info(self.base, find_by_uname="r0")
                         widget_control, uncclear3, sensitive=0
                         uncclear4=widget_info(self.base, find_by_uname="dr0")
                         widget_control, uncclear4, sensitive=1
                       endif
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;    FOURRIER SESSION 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO Getfilter, nid, slice, dr0

  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 
  common mmodes,ws,phi,dx,dy,dz,dxx,dyy,dzz
  common filter,x,y,z,nx,ny,nz,yy,zz
  common filter, ownfilter
  common choice, choice

      e1data=getdata('e1', nid)
      print, 'HDF file read: File ID = ', nid
                            
      x = DimScaleX
      y = DimScaleY
      z = DimScaleZ
      ;
      nx = n_elements(x)
      ny = n_elements(y)
      nz = n_elements(z)
      ;
      dx = fltarr(nx) & dy = fltarr(ny) & dz = fltarr(nz)
      dx[1:nx-2] = (x[2:nx-1]-x[0:nx-3])/2 & dx[0] = dx[1] & dx[nx-1] = dx[nx-2]
      dy[1:ny-2] = (y[2:ny-1]-y[0:ny-3])/2 & dy[0] = dy[1] & dy[ny-1] = dy[ny-2]
      dz[1:nz-2] = (z[2:nz-1]-z[0:nz-3])/2 & dz[0] = dz[1] & dz[nz-1] = dz[nz-2]
      dyy = rebin(reform(dy,ny,1),ny,nz)
      dzz = rebin(reform(dz,1,nz),ny,nz)
      ;
      iimag=complex(0,1)
      ;
      yy = rebin(reform(y,ny,1),ny,nz)
      zz = rebin(reform(z,1,nz),ny,nz)
      rr = sqrt(yy^2+zz^2)
      phi = atan(zz,yy)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;       Define the filter
;   (0=Sin-square, 1=Gaussian)
;
;	First find r0 
;        ... and dr0 (when multiple files selected!)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      EMax = Max(e1data(slice,*,*),I)
      IY   = I mod size(DimScaleY,/dimensions)
      IZ   = I/size(DimScaleZ,/dimensions)
      Ir0  = sqrt(DimScaleY(IY)^2 + DimScaleZ(IZ)^2)
      r0   = Ir0[0]
      if choice eq 0 then begin
       ws = cos(!pi/2*(rr-r0)/dr0)^2*(rr gt r0-dr0)*(rr lt r0+dr0)
      endif
      if choice eq 1 then begin
       ws = exp(-0.5*(rr-r0)/dr0)^2*(rr gt r0-dr0)*(rr lt r0+dr0)
      endif
      if choice eq 2 then begin
       ws = ownfilter
      endif
END

;;;..................................................................
;;;
;;; mmodes.pro
;;;
;;; A first try in getting angular information from Cartesian data
;;; Authors:  Wolfgang & Rachid
;;; 17-Jan-2000
;;; Modified: Rachid
;;; June-2001
;;;
function mmodes, f, N_m, MM=mm

common mmodes,ws,phi,dx,dy,dz,dxx,dyy,dzz

      g = reform(f)
      iimag = complex(0,1)
      mm = indgen(N_m)-N_m/2
      a = mm*complex(0,0)  ; Define A as array of same dim as M

    for i=0,n_elements(mm)-1 do begin
      m = mm[i]
      a[i] = 1/(2*!pi)*total(g*ws*exp(-iimag*m*phi)*dyy*dzz)
    endfor
    return, a

END


PRO StartFourier, nid , n_m, n_k
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;        Start FFT	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 
  common mmodes,ws,phi,dx,dy,dz,dxx,dyy,dzz
  common filter,x,y,z,nx,ny,nz,yy,zz

      e1data=getdata('e1', nid)

      print,'FFT in progress on File ID =  :', nid
      pfft = complexarr(nx,ny,nz)
      for j=0,ny-1 do begin
         for k=0,nz-1 do begin
             pfft[*,j,k] = fft(e1data[*,j,k])
         endfor
      endfor
      ;
      Lx = (x[nx-1]-x[0])*nx/(nx-1)
      a = complexarr(N_m,N_k)

      for ik=0, N_k-1 do begin
        a[*,ik] = mmodes(pfft[ik,*,*],N_m,MM=mm)
      endfor
       kk    = 2*!pi/Lx*findgen(N_k)
       a     = abs(a)
       a_cut = a & a_cut[(N_m-1)/2,0] = 0
       mmm   = mm
      
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    VISUALIZATION SESSION 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO closeEvent, event
  Widget_Control, event.top, /DESTROY
END

PRO fourier::output, event
  Widget_control, event.top,get_uvalue=uval
  widget_control, uval(20), set_value='ugh', /append
END 

PRO fourier::modeampli,event

  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 
  mvaluetext = WIDGET_INFO(event.top, FIND_BY_UNAME="m_value")
  widget_control,mvaluetext,get_value=m_value
  m_value=uint(m_value[0])
  xplot, kk, a[(N_m-1)/2+m_value,*]/a[(N_m-1)/2,*], linestyle=0, Xtitle='k_r', Ytitle='a(m)/a(m=0)', Title='m=+'+strtrim(string(m_value),1)
  xplot, kk, a[(N_m-1)/2-m_value,*]/a[(N_m-1)/2,*], linestyle=1, Xtitle='k_r', Ytitle='a(m)/a(m=0)', Title='m=-'+strtrim(string(m_value),1)

END

PRO fourier::amplitudep
;
; if "Show Mode Amplitude" is chosen
;
  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 
  common time
  common colors, b_curr, b_orig, g_curr, g_orig, r_curr, r_orig
;
; Read input from widgets
;
      widget_control,widget_info(self.inbase, find_by_uname='bid') ,get_value=bid
      bid = uint(bid[0])
      widget_control,widget_info(self.inbase, find_by_uname='eid'),get_value=eid
      eid = uint(eid[0])
      widget_control,widget_info(self.inbase, find_by_uname='sid'),get_value=sid
      sid = uint(sid[0])
      widget_control,widget_info(self.inbase, find_by_uname='slice'),get_value=slice
      slice = uint(slice[0])
      widget_control,widget_info(self.inbase, find_by_uname='n_k'),get_value=N_k
      N_k = uint(N_k[0])
      widget_control,widget_info(self.inbase, find_by_uname='m'),get_value=m
      m_value = uint(m[0])
      widget_control,widget_info(self.inbase, find_by_uname='r0'),get_value=r0
      r0 = float(r0[0])
      widget_control,widget_info(self.inbase, find_by_uname='dr0'),get_value=dr0
      dr0 = float(dr0[0])

      N_m = 2*abs(m_value)+1

      print, 'Amplitude plots (m > 0) in progress ...' 
;
; fourier uses panel
;
      oPanel = Obj_New('grpanel',columns=n_columns,rows=n_rows)
      IF self.filelist eq 0 THEN BEGIN
      eid=bid
      oPanel = Obj_New('grpanel',columns=1,rows=1) 
      ENDIF

      FOR nid = bid, eid, sid DO BEGIN
      time = nid*dt
      StartFourier, nid, n_m, n_k

      thisView = OBJ_NEW('IDLgrView', Color=[80,80,80], Viewplane_Rect=[0,0,1,1])
      thisModel = OBJ_NEW('IDLgrModel')
      thisView->Add, thisModel
      LoadCT, 33, /Silent
      TVLCT, red, green, blue, /Get
;
; set font. smaller if in a panel
;
      if bid eq eid then $
        font=obj_new('idlgrfont', 'Helvetica', size=12) $
      else $
        font=obj_new('idlgrfont', 'Helvetica', size=18/n_rows)
      xTitle = Obj_New('IDLgrText', 'k_r', Color=[255,255,0], font=font)
      yTitle = Obj_New('IDLgrText', 'a(m)/a(m=0)', Color=[255,255,0], font=font)
      plotTitle = Obj_New('IDLgrText', '[t='+strtrim(string(time),1)+' |m|= '+strtrim(string(m_value),1)+']', Color=[0,255,0], $
                    Alignment=0.5, Location=[0.475, 0.9, 0.0], font=font)
      thisModel->Add, plotTitle

      data1   = a[(N_m-1)/2+m_value,*]/a[(N_m-1)/2,*]
      data2   = a[(N_m-1)/2-m_value,*]/a[(N_m-1)/2,*]
      mindata1 = min(data1) 
      maxdata1 = max(data1) 
      mindata2 = min(data2) 
      maxdata2 = max(data2) 
      mindata  = min(mindata1,mindata2)
      maxdata  = min(maxdata1,maxdata2)
      thisPlot2 = Obj_New('IDLgrPlot', kk,data2, /histogram,$
        Color=[0,255,0], LineStyle=1, Thick=2)
      thisPlot2->GetProperty, XRange=xrange, YRange=yrange
      xs = Normalize(xrange, Position=[0.3,0.85])
      ys = Normalize(yrange, Position=[0.2,0.85])
      thisPlot2->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys
      thisModel->Add, thisPlot2
      thisPlot1 = Obj_New('IDLgrPlot', kk,data1, /histogram,$
        Color=[0,255,0], LineStyle=0, Thick=2)
      thisPlot1->GetProperty, XRange=xrange, YRange=yrange
      xs = Normalize(xrange, Position=[0.3,0.85])
      ys = Normalize(yrange, Position=[0.2,0.85])
      thisPlot1->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys
      thisModel->Add, thisPlot1

      xrange = [Min(kk), Max(kk)]
      yrange = [mindata,maxdata]
      zrange = [mindata,maxdata]

      xAxis1 = Obj_New("IDLgrAxis", 0, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, Title=xtitle, Range=xrange, XCoord_Conv=xs, $
                        Location=[1000, 0.2, 0.0], /Exact) 
      yAxis1 = Obj_New("IDLgrAxis", 1, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, Title=ytitle, Range=yrange, YCoord_Conv=ys, $
               Location=[0.3, 1000, 0.0], /Exact)
      xaxis1->getproperty, ticktext=ticktext
      ticktext->setproperty, font=font
      yaxis1->getproperty, ticktext=ticktext
      ticktext->setproperty, font=font

      thisModel->Add, xAxis1
      thisModel->Add, yAxis1
      oPanel->Add, thisView

      ENDFOR 

      self->resize, xstretch=n_columns/float(n_rows)
      self->draw, opanel

      print, '... amplitude plots (m > 0) done.' 
    end

PRO fourier::kvsm
;
; if "Show (k-m) contours" is chosen
;
  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 
  common time
  common colors, b_curr, b_orig, g_curr, g_orig, r_curr, r_orig
;
; Read input from widgets
;
      widget_control,widget_info(self.inbase, find_by_uname='bid') ,get_value=bid
      bid = uint(bid[0])
      widget_control,widget_info(self.inbase, find_by_uname='eid'),get_value=eid
      eid = uint(eid[0])
      widget_control,widget_info(self.inbase, find_by_uname='sid'),get_value=sid
      sid = uint(sid[0])
      widget_control,widget_info(self.inbase, find_by_uname='slice'),get_value=slice
      slice = uint(slice[0])
      widget_control,widget_info(self.inbase, find_by_uname='n_k'),get_value=N_k
      N_k = uint(N_k[0])
      widget_control,widget_info(self.inbase, find_by_uname='m'),get_value=m
      m_value = uint(m[0])
      widget_control,widget_info(self.inbase, find_by_uname='r0'),get_value=r0
      r0 = float(r0[0])
      widget_control,widget_info(self.inbase, find_by_uname='dr0'),get_value=dr0
      dr0 = float(dr0[0])

      N_m = 2*abs(m_value)+1

      print, '(k-m) contours in progress ...' 

      oPanel = Obj_New('grpanel',columns=n_columns,rows=n_rows)
      IF self.filelist  eq 0 THEN BEGIN
      eid=bid
      oPanel = Obj_New('grpanel',columns=1,rows=1) 
      ENDIF
      FOR nid = bid, eid, sid DO BEGIN
      time = nid*dt

      StartFourier, nid, n_m, n_k

      nlevels   = 30
      data      = a_cut 
      xtitle    = 'm'
      ytitle    = 'k_r'
      plottitle = 'Time='+strtrim(string(time),1)

      thisView = OBJ_NEW('IDLgrView', Color=[80,80,80], Viewplane_Rect=[0,0,1,1])
      thisModel = OBJ_NEW('IDLgrModel')
      thisView->Add, thisModel
      LoadCT, 33, /Silent
      TVLCT, red, green, blue, /Get
      step = (Max(data) - Min(data)) / nlevels
      levels = Indgen(nlevels) * step + Min(data)
      colorStep = Fix(255/Float(nlevels))
      cLevelColor = BIndGen(nlevels) * colorStep
      contourColors = BIndGen(3, nlevels)
      FOR j=0,nlevels-1 DO contourColors[*,j] = [red(cLevelColor[j]), $
         green(cLevelColor[j]), blue(cLevelColor[j])]
      thisContour = Obj_New('IDLgrContour', data, mmm, kk, GeomZ=-0.01, $
         Color=[255, 255, 0], /Planar, C_Color=contourColors, $
              C_Value=[levels, Max(data)],/Fill)
      thisContour->GetProperty, XRange=xrange, YRange=yrange
      xs = Normalize(xrange, Position=[0.2,0.75])
      ys = Normalize(yrange, Position=[0.2,0.85])
      thiscontour->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys

      thisModel->Add, thisContour

      if eid ne bid then $ 
         ofont = obj_new('idlgrfont', 'Helvetica', size=14/n_rows[0]) $
      else $
        ofont = obj_new('idlgrfont', 'Helvetica', size=14)
 
      xTitle = Obj_New('IDLgrText', xtitle, Color=[255,255,0], font=ofont)
      yTitle = Obj_New('IDLgrText', ytitle, Color=[255,255,0], font=ofont)
      plotTitle = Obj_New('IDLgrText', plotTitle, Color=[0,255,0], $
                    Alignment=0.5, Location=[0.475, 0.9, 0.0], font=ofont)
      thisModel->Add, plotTitle
      xrange = [Min(mmm), Max(mmm)]
      yrange = [Min(kk), Max(kk)]
      zrange = [Min(data), Max(data)]
      xs = Normalize(xrange, Position=[0.2,0.75])
      ys = Normalize(yrange, Position=[0.2,0.85])
      xAxis1 = Obj_New("IDLgrAxis", 0, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, Title=xtitle, Range=xrange, XCoord_Conv=xs, $
                        Location=[1000, 0.2, 0.0], /Exact) 
      xAxis2 = Obj_New("IDLgrAxis", 0, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, /NoText, Range=xrange, XCoord_Conv=xs, $
               Location=[1000, 0.85, 0.0], TickDir=1, /Exact)
      yAxis1 = Obj_New("IDLgrAxis", 1, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, Title=ytitle, Range=yrange, YCoord_Conv=ys, $
               Location=[0.2, 1000, 0.0], /Exact)
      yAxis2 = Obj_New("IDLgrAxis", 1, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, /NoText, Range=yrange, YCoord_Conv=ys, $
               Location=[0.75, 1000, 0.0], TickDir=1, /Exact)
      xaxis1->getproperty, ticktext=ticktext
      ticktext->setproperty, font=ofont
      yaxis1->getproperty, ticktext=ticktext
      ticktext->setproperty, font=ofont
      thisModel->Add, xAxis1
      thisModel->Add, xAxis2
      thisModel->Add, yAxis1
      thisModel->Add, yAxis2
      oPanel->Add, thisView

      ENDFOR 

      self->resize, xstretch=n_columns/float(n_rows)
      self->draw, opanel

      print, '... (k-m) contours done.' 
    end


PRO fourier::showfilter
;
; If "Check Slice and Filter" is chosen
;
  COMMON setupvariables
  COMMON fourier_data, rr, kk, mmm, a, a_cut 
  common time
  common colors, b_curr, b_orig, g_curr, g_orig, r_curr, r_orig

      widget_control,widget_info(self.inbase, find_by_uname='bid') ,get_value=bid
      bid = uint(bid[0])
      widget_control,widget_info(self.inbase, find_by_uname='eid'),get_value=eid
      eid = uint(eid[0])
      widget_control,widget_info(self.inbase, find_by_uname='sid'),get_value=sid
      sid = uint(sid[0])
      widget_control,widget_info(self.inbase, find_by_uname='slice'),get_value=slice
      slice = uint(slice[0])
      widget_control,widget_info(self.inbase, find_by_uname='n_k'),get_value=N_k
      N_k = uint(N_k[0])
      widget_control,widget_info(self.inbase, find_by_uname='m'),get_value=m
      m_value = uint(m[0])
      widget_control,widget_info(self.inbase, find_by_uname='r0'),get_value=r0
      r0 = float(r0[0])
      widget_control,widget_info(self.inbase, find_by_uname='dr0'),get_value=dr0
      dr0 = float(dr0[0])

      N_m = 2*abs(m_value) + 1

      print,'Visualizing Slice and Filter ...'

      oPanel = Obj_New('grpanel',columns=n_columns,rows=n_rows)
      IF self.filelist eq 0 THEN BEGIN
      eid=bid
      oPanel = Obj_New('grpanel',columns=1,rows=1) 
      ENDIF
      FOR nid = bid, eid, sid DO BEGIN

      time = nid*dt

      Getfilter, nid, slice, dr0
      e1data=getdata('e1',nid)
      nlevels   = 30
      data      = REFORM(e1data(slice,*,*)) 
      xtitle    = 'X2'
      ytitle    = 'X3'
      plottitle = 'Time='+strtrim(string(time),1)

      thisView = OBJ_NEW('IDLgrView', Color=[80,80,80], Viewplane_Rect=[0,0,1,1])
      thisModel = OBJ_NEW('IDLgrModel')

      LoadCT, 33, /Silent
      TVLCT, red, green, blue, /Get
      step = (Max(data) - Min(data)) / nlevels
      levels = Indgen(nlevels) * step + Min(data)
      colorStep = Fix(255/Float(nlevels))
      cLevelColor = BIndGen(nlevels) * colorStep
      contourColors = BIndGen(3, nlevels)
      FOR j=0,nlevels-1 DO contourColors[*,j] = [red(cLevelColor[j]), $
         green(cLevelColor[j]), blue(cLevelColor[j])]
      thisContour = Obj_New('IDLgrContour', data, geomx=DimScaleY, geomy=DimScaleZ, GeomZ=-0.01, $
         Color=[255, 255, 0], /Planar, C_Color=contourColors, $
              C_Value=[levels, Max(data)],/Fill)
      thisContour->GetProperty, XRange=xrange, YRange=yrange
      xs = Normalize(xrange, Position=[0.2,0.75])
      ys = Normalize(yrange, Position=[0.2,0.85])
      thiscontour->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys

      thatContour = Obj_New('IDLgrContour', rr, geomx=DimScaleY, geomy=DimScaleZ, GeomZ=-0.01, $
       /Planar,C_Value=[r0-dr0,r0-dr0/2,r0,r0+dr0/2,r0+dr0],C_LINESTYLE=[2,0,0,0,2])
      thatContour->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys
      thisModel->Add, thatContour
      thisModel->Add, thisContour

      if eid ne bid then $ 
         ofont = obj_new('idlgrfont', 'Helvetica', size=14/n_rows[0]) $
      else $
        ofont = obj_new('idlgrfont', 'Helvetica', size=14)
      xTitle = Obj_New('IDLgrText', xtitle, Color=[255,255,0], font=ofont)
      yTitle = Obj_New('IDLgrText', ytitle, Color=[255,255,0], font=ofont)
      plotTitle = Obj_New('IDLgrText', plotTitle, Color=[0,255,0], $
                    Alignment=0.5, Location=[0.475, 0.9, 0.0], font=ofont)
      thisModel->Add, plotTitle
      xrange = [Min(DimScaleY), Max(DimScaleY)]
      yrange = [Min(DimScaleZ), Max(DimScaleZ)]
      zrange = [Min(data), Max(data)]
      xs = Normalize(xrange, Position=[0.2,0.75])
      ys = Normalize(yrange, Position=[0.2,0.85])
      xAxis1 = Obj_New("IDLgrAxis", 0, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, Title=xtitle, Range=xrange, XCoord_Conv=xs, $
                        Location=[1000, 0.2, 0.0], /Exact) 
      xAxis2 = Obj_New("IDLgrAxis", 0, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, /NoText, Range=xrange, XCoord_Conv=xs, $
               Location=[1000, 0.85, 0.0], TickDir=1, /Exact)
      yAxis1 = Obj_New("IDLgrAxis", 1, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, Title=ytitle, Range=yrange, YCoord_Conv=ys, $
               Location=[0.2, 1000, 0.0], /Exact)
      yAxis2 = Obj_New("IDLgrAxis", 1, Color=[255,255,0], Ticklen=0.05, $
               Minor=4, /NoText, Range=yrange, YCoord_Conv=ys, $
               Location=[0.75, 1000, 0.0], TickDir=1, /Exact)
      xaxis1->getproperty, ticktext=ticktext
      ticktext->setproperty, font=ofont
      yaxis1->getproperty, ticktext=ticktext
      ticktext->setproperty, font=ofont
      thisModel->Add, xAxis1
      thisModel->Add, xAxis2
      thisModel->Add, yAxis1
      thisModel->Add, yAxis2
      thisView->Add, thisModel
      oPanel->Add, thisView

      ENDFOR 

      self->resize, xstretch=n_columns/float(n_rows)
      self->draw, opanel
      print, 'Slice and Filter done.'
    end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    WIDGET SESSION 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FUNCTION fourier::init 

COMMON setupvariables
common time
common choice, choice

status = self->Tool::init('Fourier')

; Initializing values

  bid   = 000
  eid   = 099
  sid   = 1
;  dt    = 10.0
  slice = ROUND((DimSizes(0)-1)/3) 
  N_k   = 12
  N_m   = 5 
  r0    = 0.0
  dr0   = 3.0
  time  = dt*bid
  choice= 0

  filelist = FSC_Droplist(self.inbase, Value=['Single-File', 'Multiple-Files'], $
    Index=0, uname='fourier_filelist', UValue='fourier_filelist' ,title='File(s)   ')

  firsttext = cw_field(self.inbase, value=bid, uname='bid', title='First     =')

  lasttext = cw_field(self.inbase, value=eid, uname='eid', title= 'Last      =')

  steptext = cw_field(self.inbase, value=sid, uname='sid',title=  'Increment =')

;  dttext = cw_field(self.inbase, value=dt, uname='dt', title='dt        =')

  slicetext=cw_field(self.inbase, value=slice, uname='slice', title='Slice     =')

  N_ktext=cw_field(self.inbase, value=N_k, uname='n_k', title='N_k       =')

  N_mtext=cw_field(self.inbase, value=N_m, uname='m', title='|m|       =')

  filterlabel=widget_label(self.inbase, value='Filter', /align_left)

  r0text=cw_field(self.inbase, value=r0, uname='r0',title='r0        =')

  dr0text=cw_field(self.inbase, value=dr0, uname='dr0',title='dr0       =')

;  color = widget_button(self.inbase, value='XPalette', uvalue='XPalette')

  filterlist = widget_button(self.inbase,value='Step1: Select Filter', uvalue='selectfilter')

  filtershow = widget_button(self.inbase,value='Step2: Check Slice and Filter',uvalue='showfilter')

  kvsmbutton = widget_button(self.inbase,value='Show (k-m) Contours ', uvalue='kvsm')

  showamplip = widget_button(self.inbase,value='Show Mode Amplitude ', uvalue='amplitudep')

  psbutton = widget_button(self.inbase,value='   Save   ', uvalue='ps')

  helpbutton = widget_button(self.inbase,value='HELP', uvalue='Help')

  closeButton = Widget_Button(self.inbase, Value='Close', Event_Pro='closeEvent')

  self->up,0

  widget_control, self.base, /update

  self->resize

  self.HelpFile = "help/HELP_FOURIER"

  return, 1

END

PRO fourier::set, filelist = filelist
  if n_elements(filelist) NE 0 then self.filelist=filelist
END

PRO fourier__define
   void = {fourier, filelist:0, bid:0, eid:0, sid:0, dt:1, slice:39, $
           n_k:12, m:5, r0:0, dr0:3, Vars: ptr_new(1), $ 
           INHERITS tool}
END
