;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; JETGET-tool for putting together variables read from an ASCII or HDF file.
;
; Written summer 2002 by Michael Joergensen
;
; To be called from the different JETGET modules. Will call readhdf_1.pro or
; readascii.pro.
;

FUNCTION GetData, Var, ID, Filename=Filename

  COMMON SetupVariables
  COMMON new_variables

;
; Either the structure of the datafiles are the default
; (n_elements(ownvarlist)=0), or it has been changed
; (n_elements(ownvarlist) ne 0). First find the number in the list the
; requested variable has.
;
  if n_elements(ownvarlist) ne 0 then whereinvarlist=where(var eq ownvarlist) $
  else whereinvarlist=where(var eq varlist)
;
; If it is a complex variable (not one directly readable from the datafile)
; whereinvarlist is empty and the following is skipped. In that case the 
; requested variable is hopefully found from the list underneath, and this
; part will be called again later.
;
  if whereinvarlist[0] ge 0 then begin
;
; Either read the variable from a HDF file...
;
    if filetype eq 0 then begin
     readhdf_1, id, whereinvarlist[0]+2, Data, Filename = Filename
     return, Data
    endif else begin
; 
; Or read the variable from an ASCII file.
;
     readascii, id, whereinvarlist[0]+1, Data, Filename = Filename
     return, Data
    endelse
  endif else begin
;
; If it was a complex variable (not directly readable from the datafile), it
; should be found in the list underneath:
;
  CASE Var OF
    'LOG_d' : BEGIN
                Data=getdata('d', id, filename=filename)
		Data = ALOG10(Data)
              END
    'LOG_e1' : BEGIN
                 Data=getdata('e1', id, filename=filename)
		 Data = alog10(Data)
               END
    'LOG_e2' : BEGIN
                  Data=getdata('e2', id, filename=filename)
                  Data = alog10(Data)
                END
    'KinEne' : BEGIN
                 v1data=getdata('v1', id, filename=filename)
                 v2data=getdata('v2', id, filename=filename)
                 v3data=getdata('v3', id, filename=filename)
                 ddata=getdata('d', id, filename=filename)
                 Data=DData*(V1Data^2+V2Data^2+V3Data^2)
                 V1Data = 0
                 V2Data = 0
                 V3Data = 0
                 DData = 0
               END
    'MagEne' : BEGIN
                 b1data=getdata('b1', id, filename=filename)
                 b2data=getdata('b2', id, filename=filename)
                 b3data=getdata('b3', id, filename=filename)
                 Data = B1Data^2+B2Data^2+B3Data^2
                 B1Data = 0
                 B2Data = 0
                 B3Data = 0
               END
     'V'     : BEGIN
                 v1data=getdata('v1', id, filename=filename)
                 v2data=getdata('v2', id, filename=filename)
                 v3data=getdata('v3', id, filename=filename)
                 Data = V1Data^2+V2Data^2+V3Data^2
                 V1Data = 0
                 V2Data = 0
                 V3Data = 0
               END
      'B'     : BEGIN
                 b1data=getdata('b1', id, filename=filename)
                 b2data=getdata('b2', id, filename=filename)
                 b3data=getdata('b3', id, filename=filename)
                 Data = B1Data^2+B2Data^2+B3Data^2
                 B1Data = 0
                 B2Data = 0
                 B3Data = 0
               END
      'b_p'     : BEGIN 
                  b1data=getdata('b1', id, filename=filename)
                  b2data=getdata('b2', id, filename=filename)
                  b3data=getdata('b3', id, filename=filename)
                  s=size(B2Data)
                  brData = fltarr(s[1],s[2],s[3])
                  brdata = b3data*angle1 + b2data*angle2
                  Data = reform(b1data*b1data+brdata*brdata)
                  brdata = 0
                  b1data = 0
                  b2data = 0
                  b3data = 0
               END
      'b_phi'  : BEGIN
                  b3data=getdata('b3', id, filename=filename)
                  b2data=getdata('b2', id, filename=filename)
                  s=size(B2Data)
                  Data = fltarr(s[1],s[2],s[3])
                     Data = reform(b3data)*angle2-reform(b2data)*angle1
                  b2data = 0
                  b3data = 0
               END
      'bphi'  : BEGIN
                  b3data=getdata('b3', id, filename=filename)
                  b2data=getdata('b2', id, filename=filename)
                  s=size(B2Data)
                  Data = fltarr(s[1],s[2],s[3])
                     Data = reform(b3data)*angle2-reform(b2data)*angle1
                  b2data = 0
                  b3data = 0
               END
      'b_phi/b_p' : BEGIN
                  bphi = getdata('b_phi', id, filename=filename)
                  bp = getdata('b_p', id, filename=filename)
                  data = abs(bphi/sqrt(bp))
                  bp = 0
                  bphi=0
               END
      'v_phi'  : BEGIN
                  V2Data=getdata('v2', id, filename=filename)
                  V3Data=getdata('v3', id, filename=filename)
		  s=size(V2Data)
                  Data = fltarr(s[1],s[2],s[3])
		    Data = reform(V3Data)*angle2 - Reform(V2Data)*angle1
                  V2Data = 0
                  V3Data = 0
                END
      'vphi'  : BEGIN
                  V2Data=getdata('v2', id, filename=filename)
                  V3Data=getdata('v3', id, filename=filename)
		  s=size(V2Data)
                  Data = fltarr(s[1],s[2],s[3])
		  for i=0, s[1]-1 do begin
		    Data = reform(V3Data)*angle2 - Reform(V2Data)*angle1
                  endfor
                  V2Data = 0
                  V3Data = 0
                END
      'v_p'     : BEGIN 
                  v1data=getdata('v1', id, filename=filename)
                  v2data=getdata('v2', id, filename=filename)
                  v3data=getdata('v3', id, filename=filename)
                  s=size(v2Data)
                  vrData = fltarr(s[1],s[2],s[3])
                  vrdata = v3data*angle1 + v2data*angle2
                  Data = reform(v1data*v1data+vrdata*vrdata)
                  vrdata = 0
                  v1data = 0
                  v2data = 0
                  v3data = 0
               END
    'LOG_MagEne' : BEGIN
                     b1data=getdata('b1', id, filename=filename)
                     b2data=getdata('b2', id, filename=filename)
                     b3data=getdata('b3', id, filename=filename)
                     Data = B1Data^2+B2Data^2+B3Data^2
                     B1Data = 0
                     B2Data = 0
                     B3Data = 0
                     Data = alog10(Data)
               END
    'MassFlux' : BEGIN
                   v1data=getdata('v1', id, filename=filename)
                   ddata=getdata('d', id, filename=filename)
                   Data = DData*V1Data
                   DData = 0
                   V1Data = 0
                 END
    'MomentumFlux' : BEGIN
                       v1data=getdata('v1', id, filename=filename)
                       ddata=getdata('d', id, filename=filename)
                       Data = DData*V1Data^2
                       DData = 0
                       V1Data = 0
                     END
    'EnergyFlux'   : BEGIN
                       v1data=getdata('v1', id, filename=filename)
                       ddata=getdata('d', id, filename=filename)
                       Data = DData*V1Data^3
                       DData = 0
                       V1Data = 0
                     END
    'MagneticFlux' : BEGIN
                       b1data=getdata('b1', id, filename=filename)
                       Data = B1Data
                       B1Data = 0
                     END
    'inclangle'    : Begin
                       b1data=getdata('b1', id, filename=filename)
                       b2data=getdata('b2', id, filename=filename)
                       b3data=getdata('b3', id, filename=filename)
                       Data1 = sqrt(B2data^2+B3data^2)
                       Data  = abs(atan(B1data/Data1)*180/!pi)
                       B1Data = 0
                       B2Data = 0
                       B3Data = 0
                       Data1 = 0
                     END
   ELSE : BEGIN
;
; If the requested variable was not in the list above, it should be defined
; in the vars module, and will be found underneath:
;
    whereinvarlabels = where(var eq varlabels)
    if whereinvarlabels ge 0 then begin
;
; This loop is simply to find what variables are needed to 
; read in for the wanted expression.
;
     for i=0, (n_elements(varlist)>n_elements(extravars))-1 do begin
;     for i=0,n_elements(varlist)-1 do begin
      if i le n_elements(varlist)-1 then begin
        if strpos(string(variables(whereinvarlabels)), string(varlist[i])) ge 0 then begin
          getdatastring=string(varlist[i])+"=getdata('"+string(varlist[i])+"', id)"
          execres=execute(getdatastring)
          if not(execres) then print, "ajajajaj"
        endif
      endif
      if i le n_elements(extravars)-1 then begin
        if strpos(string(variables(whereinvarlabels)), string(extravars[i])) ge 0 then begin
          getdatastring=string(extravars[i])+"=getdata('"+string(extravars[i])+"', id)"
          execres=execute(getdatastring)
          if not(execres) then print, "ajajajaj"
        endif
      endif
     endfor
      succes = execute(variables[whereinvarlabels[0]])
;
; If the variable has not been found, something must be wrong. Print an 
; error message and stop JETGET (is this too hard?):
;
      if not(succes) then begin
            print, "*** UPS, something wrong in your formula ***"
            print, varlabels[whereinvarlabels[0]], ": ", variables[whereinvarlabels[0]]
            stop  
            Data=0
;
; One can get back and running by typing retall in the stopped idl window.
;
      endif
    endif
  endelse
  endcase
endelse
  return, Data
END
