;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Contour object which expands the functionality of IDL's building object.
; To be used in JETGET.
; Created Summer 2001
;   July,  2001, M. S. Joergensen
;
; Modification history:
;
;
; Usage:
;
FUNCTION grContour::FormatTick, mark

end

FUNCTION grContour::INIT, Z, usepanel=usepanel, NumberOfLevels=NumberOfLevels, $
	XTITLE=XTITLE, YTITLE=YTITLE, TITLE=TITLE, XMin=XMin, XMax=XMax, YMin=YMin, YMax=YMax, $
	Color=Color, Palette=Palette, Max=Max, Min=Min, nocolorbar=nocolorbar, $
        BTitle=BTitle, dims1=dims1, dims2=dims2, _EXTRA = e

;
; The init function for grcontour
;
; The mandatory variable Z is the data to plot.
; usepanel=1 indicates that the panel should be used, and thus fonts
;   adjusted to that.
; NumberOfLevels is the number of contour levels to draw.
; XTitle is the title on the x axis.
; YTitle is the title on the y axis.
; Title is the title at the top.
; BTitle is the title at the bottom.
; XMin is the minimum xvalue for the plot.
; XMax is the maximum xvalue for the plot (the xaxis goes from xmin to
;   xmax).
; YMin is the minimum yvalue for the plot.
; Ymax is the maximum yvalue for the plot (the yaxis goes from ymin to
;   ymax).
; Color indicates if the plot is a filled (color) plot.
; Palette is the color palette to use if it is a filled (color) plot.
; Max is the maximum contour value.
; Min is the minimum contour value.
; nocolorbar indicates that no colorbar should be drawn.
; dims1 is the x coordinates for the geometry for which the contours
;   correspond. Similar to geomx in IdlGrContour.
; dims2 is the y coordinates for the geometry for which the contours
;   correspond. Similar to geomy in IdlGrContour.
;

common setupvariables
common colors, b_curr, b_orig, g_curr, g_orig, r_curr, r_orig
;
; Set some variables
;
    Default, NumberOfLevels, 50
    Default, C_COLORS, 0
    Default, CT, 5
    Default, Palette, OBJ_NEW('IDLgrPalette')
    default, usepanel, 0
    default, b_curr, indgen(255)
    default, g_curr, indgen(255)
    default, r_curr, indgen(255)

    IF n_elements(g_curr) ne 0 then $
        palette=obj_new('idlgrpalette', b_curr, r_curr, g_curr) 

    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0

    IF keyword_set(Color) THEN $
      C_COLORS = findgen(NumberOfLevels)*255./(NumberOfLevels)

    C_COLORS_RGB = intarr(NumberOfLevels-1,3)
    C_COLORS_R = intarr(NumberOfLevels-1)
    C_COLORS_G = intarr(NumberOfLevels-1)
    C_COLORS_B = intarr(NumberOfLevels-1)

;
; Cvalue is the values for which the contours are to be drawn
; Linestyle is the style of the line. 
; linestyle=0 is a solid line
; linestyle = 2 is a dashed line
; solid line is used for positive values, or if all values are negative
; dashed line is used for negative values, when the data contains both 
; positive and negative values.
;
    cvalue=min+findgen(numberoflevels)*(max-min)/(numberoflevels-1)  
    linestyle = fltarr(NumberofLevels)
    if ((max gt 0) and (min lt 0)) then begin 
      linestyle[where(cvalue lt 0)]=2
    endif
    
;
; Make the contour object
;
    oContour = OBJ_NEW('IDLgrContour', Z, geomx=dims1, geomy=dims2, n_levels=numberoflevels, c_linestyle=linestyle, /PLANAR, GEOMZ=-0.01, C_value=Cvalue, C_COLOR=C_COLORS, Palette=Palette, _EXTRA = e)
    s=size(Z)
;
; If using a panel, chose a different font size
;
    if (usepanel eq 1) and (n_rows gt 1)then begin
     ofont = obj_new('idlgrfont', 'Helvetica', size=14/n_rows[0]) 
     if (float(n_rows[0]) eq 1) and (float(n_columns[0]) ne 1) then $
      ofont = obj_new('idlgrfont', 'Helvetica', size=14/2.) 
    endif else $
     ofont = obj_new('idlgrfont', 'Helvetica', size=14)
    oTitle = OBJ_NEW('IDLgrText', Title,alignment=0.5, $ 
       LOCATION=[(xmax+xmin)/2,ymax+0.01*(ymax-ymin)], font=ofont)
;
; Make the colorbar
;
    oColorBar = OBJ_NEW('VColorbar', Palette=Palette, Range=[min,max], $
      Position=[xmax+(0.15*(xmax-xmin)),ymin,xmax+(0.17*(xmax-xmin)),ymax], $
      color=[0,0,0], /normal, usepanel=usepanel)
;
; The axes
;
    oX1Axis = OBJ_NEW('IDLgrAxis', 0)
    oY1Axis = OBJ_NEW('IDLgrAxis', 1)
    oX2Axis = OBJ_NEW('IDLgrAxis', 0)
    oY2Axis = OBJ_NEW('IDLgrAxis', 1)

    oX1Axis->SetProperty, RANGE=[Xmin, Xmax], LOCATION=[xmin, ymin], /EXACT
    oY1Axis->SetProperty, RANGE=[Ymin, Ymax], LOCATION=[xmin, ymin], /EXACT
    oX2Axis->SetProperty, RANGE=[Xmin, Xmax], LOCATION=[xmin, ymax], /EXACT, /NOTEXT, TICKDIR=1
    oY2Axis->SetProperty, RANGE=[Ymin, Ymax], LOCATION=[xmax, ymin], /EXACT, /NOTEXT, TICKDIR=1
;
; When using the panel, the fonts on the axes must be smaller too
;
    if (usepanel eq 1) and (n_rows gt 1)then begin
      if (float(n_rows[0]) eq 1) and (float(n_columns[0]) ne 1) then $
       ofont = obj_new('idlgrfont', 'Helvetica', size=12/2.) $
      else $
       ofont=obj_new('idlgrfont', 'Helvetica', size=12/(n_rows[0]))
      oTitlebottom = OBJ_NEW('IDLgrText', BTitle,alignment=0.5, $ 
         LOCATION=[(xmax+xmin)/2,ymin-(ymax-ymin)/7.], font=ofont)
      oXTitle = OBJ_NEW('IDLgrText', XTitle, font=ofont)
      oTitleside = obj_new('IDLgrText', yTitle, font=ofont)
      ox1axis->setproperty, title=oxtitle
      oy1axis->setproperty, title=otitleside
    endif else begin
      ofont=obj_new('idlgrfont', 'Helvetica', size=12)
      oTitlebottom = OBJ_NEW('IDLgrText', BTitle,alignment=0.5, $ 
         LOCATION=[(xmax+xmin)/2,ymin-(ymax-ymin)/7.], font=ofont)
      oXTitle = OBJ_NEW('IDLgrText', XTitle, font=ofont)
      oTitleside = obj_new('IDLgrText', yTitle, font=ofont)
      ox1axis->setproperty, title=oxtitle
      oy1axis->setproperty, title=otitleside
    endelse
;
; Set the size of the ticks to 5% of the total plot window.
;
    ticklenx=(ymax-ymin)*0.05
    tickleny=(xmax-xmin)*0.05
    ox1axis->getproperty, ticktext=ticktext
    ticktext->setproperty, font=ofont
    ox1axis->setproperty, ticklen=ticklenx
    ox2axis->setproperty, ticklen=ticklenx
    oy1axis->getproperty, ticktext=ticktext
    ticktext->setproperty, font=ofont
    oy1axis->setproperty, ticklen=tickleny
    oy2axis->setproperty, ticklen=tickleny
;
; Put the object together
;
    Self->Add, otitlebottom
    Self->Add, oTitle
    Self->Add, oContour
    Self->Add, oX1Axis
    Self->Add, oY1Axis
    Self->Add, oX2Axis
    Self->Add, oY2AxiS
    if not keyword_set(nocolorbar) then Self->Add, oColorBar

    thiscontainer=obj_new('idl_container')
    thiscontainer->add, ocontour    
    thiscontainer->add, ox1axis
    thiscontainer->add, ox2axis
    thiscontainer->add, oy1axis
    thiscontainer->add, oy2axis
    if keyword_set(color) then thiscontainer->Add, oColorBar

    self.ocontour=ocontour
    self.ox1axis=ox1axis
    self.ox2axis=ox2axis
    self.oy1axis=oy1axis
    self.oy2axis=oy2axis
    if keyword_set(color) then self.ocolorbar=ocolorbar
    self.thiscontainer=thiscontainer
    return, 1
END

PRO grcontour::cleanup
;
; A cleanup routine for grcontour.
;
   obj_destroy, self.thiscontainer
   self->idlgrmodel::cleanup
END

PRO grContour__define

  struct = { grContour, INHERITS IDLgrModel, $
		oX1Axis: OBJ_NEW(), oX2Axis: OBJ_NEW(), oY1Axis: OBJ_NEW(), oY2Axis: OBJ_NEW(), $
		oContour: OBJ_NEW(), oColorBar: OBJ_NEW(), nAxis: 4, $
	        thiscontainer: OBJ_NEW()}

END
