;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; $Id: grfieldlines__define.pro,v 1.11 2003/07/02 16:45:23 staff Exp $
;
; Based on flow3
;
; To be used by the JETGET modules.
;
;
; The index2x/y/zUnit converts between locations in the grid and 
; physical coordinates
;
FUNCTION grFieldlines::index2xunit, x

  s = n_elements(x)
  t = n_elements(*Self.DimScaleX)

  res = fltarr(s)

  for i=0L,s-1 do begin

    res[i]=(x[i]-fix(x[i]))*((*self.dimscalex)[x[i]+1]-(*self.dimscalex)[x[i]])+(*self.dimscalex)[x[i]]

  endfor
  return, res
END
FUNCTION grFieldlines::index2yunit, y

  s = n_elements(y)
  t = n_elements(*Self.DimScaleY)

  res = fltarr(s)

  for i=0L,s-1 do begin

    res[i]=(y[i]-fix(y[i]))*((*self.dimscaley)[y[i]+1]-(*self.dimscaley)[y[i]])+(*self.dimscaley)[y[i]]

  endfor
  return, res
END
FUNCTION grFieldlines::index2zunit, z

  s = n_elements(z)
  t = n_elements(*Self.DimScaleZ)

  res = fltarr(s)

  for i=0L,s-1 do begin

    res[i]=(z[i]-fix(z[i]))*((*self.dimscalez)[z[i]+1]-(*self.dimscalez)[z[i]])+(*self.dimscalez)[z[i]]

  endfor
  return, res
END
;
; The x/y/zUnit2Index converts between physical coordinates and location in
; the grid
;
FUNCTION grFieldlines::xUnit2Index, x

  s = n_elements(x)
  t = n_elements(*Self.DimScaleX)

  res = fltarr(s)

  for i=0L,s-1 do begin
     minv = min(abs(x[i]-*Self.DimScaleX),minp)

     if (((*Self.DimScaleX)[minp] GT x[i]) and (minp GT 0)) then $
       minp = minp - 1

     if (minp ge t-2) then $
       minp = t-2

     res[i] = minp+(x[i]-(*Self.DimScaleX)[minp])/((*Self.DimScaleX)[minp+1]-(*Self.DimScaleX)[minp])
  endfor

  return, res
END

FUNCTION grFieldlines::yUnit2Index, y

  s = n_elements(y)
  t = n_elements(*Self.DimScaleY)

  res = fltarr(s)

  for i=0L, s-1 do begin
     minv = min(abs(Y[i]-*Self.DimScaleY),minp)

     if (((*Self.DimScaleY)[minp] GT y[i]) and (minp GT 0)) then $
       minp = minp - 1

     if (minp ge t-2) then $
       minp = t-2

     res[i] = minp+(y[i]-(*Self.DimScaleY)[minp])/((*Self.DimScaleY)[minp+1]-(*Self.DimScaleY)[minp])
  endfor

  return, res
END

FUNCTION grFieldlines::zUnit2Index, z

  s = n_elements(z)
  t = n_elements(*Self.DimScaleZ)

  res = fltarr(s)

  for i=0L, s-1 do begin
     minv = min(abs(z[i]-*Self.DimScaleZ),minp)

     if (((*Self.DimScaleZ)[minp] GT z[i]) and (minp GT 0)) then $
       minp = minp - 1

     if (minp ge t-1) then $
       minp = t-2

     res[i] = minp+(z[i]-(*Self.DimScaleZ)[minp])/((*Self.DimScaleZ)[minp+1]-(*Self.DimScaleZ)[minp])
  endfor

  return, res

END
;
; The following Get-functions simply returns a min or max value
;
FUNCTION grFieldlines::GetX1Min

  RETURN, Self.x1min
END

FUNCTION grFieldlines::GetX2Min

  RETURN, Self.x2min
END

FUNCTION grFieldlines::GetX3Min

  RETURN, Self.x3min
END

FUNCTION grFieldlines::GetX1Max

  RETURN, Self.x1max
END

FUNCTION grFieldlines::GetX2Max

  RETURN, Self.x2max
END

FUNCTION grFieldlines::GetX3Max

  RETURN, Self.x3max
END

PRO grFieldlines::GetProperty, Fieldline=Fieldline, Number=Number, Sy=Sy, Sz=Sz, indicies=indicies, _REF_EXTRA=e
;
; getproperty routine
;
  IF (ARG_PRESENT(Fieldline)) THEN BEGIN
    IF ((N_ELEMENTS(Sy) NE 0) AND (N_ELEMENTS(Sz) NE 0)) THEN BEGIN
      FOR I=0, Self->IDL_Container::Count() - 1 DO BEGIN
        Tmp = Self->Idl_Container::Get(POSITION=I)
        TMP->GetProperty, Data=Data
        IF (Data[1,0] EQ Sy) AND (Data[2,0] EQ Sz) THEN $
	  FieldLineData=Data
      ENDFOR
    END ELSE BEGIN
      Line = Self->IDL_Container::Get()
      Line->GetProperty, Data=FieldlineData
      OBJ_DESTROY, Line
    END
  END

  IF (ARG_PRESENT(Fieldline)) THEN BEGIN
   if (keyword_set(indicies)) then begin
     n = n_elements(FieldlineData[0,*])
     Fieldline = fltarr(3,n)
     Fieldline[0,*] = Self->xUnit2Index(FieldlineData[0,*])
     Fieldline[1,*] = Self->yUnit2Index(FieldlineData[1,*])
     Fieldline[2,*] = Self->zUnit2Index(FieldlineData[2,*])
   end else begin
     Fieldline = FieldlineData
   end
  end

  IF (n_elements(e) ne 0) then $
    Self->IdlgrModel::GetProperty, _EXTRA=e
END

FUNCTION grFieldlines::CalcLine, Vx, Vy, Vz, Sx=Sx, Sy=Sy, Sz=Sz, nSteps=nSteps, steplen=steplen, nVecs=nVecs, backwards=backwards

common startpoints, xa, ya, za
;
; Routine that calculates the field lines
;
  notbefore=0
  s = size(vx)
  if s[0] ne 3 then message,'grfieldlines: Vx, Vy, and Vz must be 3D arrays'
  nx = s[1]
  ny = s[2]
  nz = s[3]
  if n_elements(nsteps) le 0 then nsteps = (nx > ny > nz) /5

;
; find starting points
;
  IF n_elements(sx) le 0 THEN BEGIN ;Starting points specified?
    IF n_elements(nvecs) le 0 THEN nvecs = 200
    x1 = REPLICATE(3., nVecs)
    y1 = randomu(seed, nVecs) * (ny-1)
    z1 = randomu(seed, nVecs) * (nz-1)
  ENDIF ELSE BEGIN
    x1 = float(sx)
    y1 = float(sy)
    z1 = float(sz)
    nvecs = n_elements(x1) < n_elements(y1) < n_elements(z1)
  ENDELSE
;
; if interpolating backwards, use previous end points as starting points
;
  if keyword_set(backwards) then begin
    x1=xa
    y1=ya
    z1=za
  endif

  flags = (Self->xUnit2Index(x1) ge 0) and (Self->xUnit2Index(x1) le nx-1) and (Self->yUnit2Index(y1) ge 0) $
    and (Self->yUnit2Index(y1) le ny-1) and (Self->zUnit2Index(z1) ge 0) and (Self->zUnit2Index(z1) le nz-1)

  mypolyline = OBJARR(nVecs)

  x1tab=fltarr(nvecs,nsteps+1)
  x2tab=fltarr(nvecs,nsteps+1)
  x3tab=fltarr(nvecs,nsteps+1)
;
; find the direction of the fieldlines
;
  sign = interpolate(vx, Self->xUnit2Index(x1), Self->yUnit2Index(y1), Self->zUnit2Index(z1))/$
    abs(interpolate(vx, Self->xUnit2Index(x1), Self->yUnit2Index(y1), Self->zUnit2Index(z1)))

  if keyword_set(backwards) then sign=-sign

  scale = fltarr(nvecs)

  Self.x1min = min(x1)
  Self.x1max = max(x1)
  Self.x2min = min(y1)
  Self.x2max = max(y1)
  Self.x3min = min(z1)
  Self.x3max = max(z1)

  FOR i=0L,nsteps DO BEGIN      ;Draw each element
    x0 = x1
    y0 = y1
    z0 = z1
;
; the step size
;
    dx1 = sign*interpolate(vx, Self->xUnit2Index(x0), Self->yUnit2Index(y0), Self->zUnit2Index(z0))
    dx2 = sign*interpolate(vy, Self->xUnit2Index(x0), Self->yUnit2Index(y0), Self->zUnit2Index(z0))
    dx3 = sign*interpolate(vz, Self->xUnit2Index(x0), Self->yUnit2Index(y0), Self->zUnit2Index(z0))

    for l=0, nvecs-1 do $
      scale[l] = steplen/sqrt(dx1[l]^2+dx2[l]^2+dx3[l]^2)

    x1 = dx1*scale + x0
    y1 = dx2*scale + y0
    z1 = dx3*scale + z0
    flags = flags and $
      (Self->xUnit2Index(x1) ge 0) and (Self->xUnit2Index(x1) le nx-1) and $
      (Self->yUnit2Index(y1) ge 0) and (Self->yUnit2Index(y1) le ny-1) and $
      (Self->zUnit2Index(z1) ge 0) and (Self->zUnit2Index(z1) le nz-1)

    FOR j=0,nvecs-1 DO $
      IF flags[j] THEN BEGIN
      Self.x1min = min([Self.x1min,x0[j]])
      Self.x1max = max([Self.x1max,x0[j]])
      Self.x2min = min([Self.x2min,y0[j]])
      Self.x2max = max([Self.x2max,y0[j]])
      Self.x3min = min([Self.x3min,z0[j]])
      Self.x3max = max([Self.x3max,z0[j]])
      x1tab[j,i]=x0[j]
      x2tab[j,i]=y0[j]
      x3tab[j,i]=z0[j]
    ENDIF 
        xa=x0
        ya=y0
        za=z0
  ENDFOR

  FOR k=0,nVecs-1 DO BEGIN
    indicies = WHERE((Self->xUnit2Index(x1tab[k,*]) GT 0) AND (Self->yUnit2Index(x2tab[k,*]) GT 0) AND (Self->zUnit2Index(x3tab[k,*]) GT 0))
    IF (N_ELEMENTS(indicies) NE 1) THEN $
      mypolyline[k] = OBJ_NEW('IDLgrPolyline',x1tab[k,indicies],x2tab[k,indicies],x3tab[k,indicies]) 
  ENDFOR
  xa=x1
  ya=y1
  za=z1
  RETURN, myPolyline
END

FUNCTION grFieldLines::Init, Vx, Vy, Vz, Sx=Sx, Sy=Sy, Sz=Sz, nVecs=nVecs, nSteps=nSteps, steplen=steplen, DimScaleX=DimScaleX, DimScaleY=DimScaleY, DimScaleZ=DimScaleZ, backwards=backwards

  IF (Self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0

  Self.DimScaleX = ptr_new(1.)
  Self.DimScaleY = ptr_new(1.)
  Self.DimScaleZ = ptr_new(1.)

  s = size(Vx)
;
; some security check. shouldn't be necessary
;
  if n_elements(DimScaleX) eq 0 then $
    *Self.DimScaleX = findgen(s[1]) $
  else $
    *Self.DimScaleX = DimScaleX

  if n_elements(DimScaleY) eq 0 then $
    *Self.DimScaleY = findgen(s[2]) $
  else $
    *Self.DimScaleY = DimScaleY

  if n_elements(DimScaleZ) eq 0 then $
    *Self.DimScaleZ = findgen(s[3]) $
  else $
    *Self.DimScaleZ = DimScaleZ
;
; make the objects
;
  oFieldlines = Self->CalcLine(Vx, Vy, Vz, Sx=Sx, Sy=Sy, Sz=Sz, nSteps=nSteps, steplen=steplen, nVecs=nVecs, backwards=backwards)

;  oJetAxis = OBJ_NEW('IDLgrPolyline',[0.5,0.5],[0,0],[0,DimScaleX[n_elements(DimScaleX) - 1]],color=[255,0,0])
  oJetAxis = OBJ_NEW('IDLgrPolyline',[0,(*Self.DimScaleX)[s[1]-1]],[0,0],[0,0],color=[255,0,0])

  Self->Add, oJetAxis

  FOR k=0L,nVecs-1 DO BEGIN
    IF (OBJ_VALID(oFieldlines[k])) THEN $
      Self->Add, oFieldlines[k]
  ENDFOR

  RETURN, 1
END

PRO grFieldlines__define
  struct = { grFieldlines, INHERITS IDLgrModel, oJetAxis: OBJ_NEW(), data: PTR_NEW(), $
             DimScaleX:ptr_new(), DimScaleY:ptr_new(), DimScaleZ:ptr_new(), x1min:0., x2min:0., $
             x3min:0., x1max:0., x2max:0., x3max:0. }
END
