;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Panel object which expands the functionality of IDL's buildin object.
; Created Summer 2001
;   July,  2001, M. S. Joergensen
;
; Modification history:
;
; To be used by the different JETGET modules
;

PRO grPanel::Reset
  Self.Col = 1
  Self.Row = 1
END

PRO grPanel::Add, oView, nolocation=nolocation, units=units, dimensions=dimensions
    default, units,3
    default, dimensions, [self.dx, self.dy]
;
; If the panel is not big enough for the requested views, clean up the panel
; and make a new one starting with an image in the upper left corner.
;
    IF (self.Row GT Self.NRow) then begin
      ncol=self.ncol
      nrow=self.nrow
      Self.Row = 1
      obj_destroy, self
      self=obj_new('grpanel',columns=ncol, rows=nrow)
    end
;
; Make a view holding the object to be put into the panel
;
    tmpView = oView
    tmpView->SetProperty, Units=units
    tmpView->SetProperty, Dimensions=dimensions
    if not keyword_set(nolocation) then $
    tmpView->SetProperty, Location=[(self.Col-1)*self.dx, 1.-self.dy*self.Row]
;
; Put it into the panel
;
    self->IDLgrScene::Add, tmpView
;
; Increase the location by one for the next view
;
    IF (self.Col EQ self.NCol) THEN BEGIN
      self.Row = self.Row + 1
      self.Col = 1
    END ELSE $
      self.Col = self.Col + 1

END

PRO grpanel::cleanup

      obj_destroy, self
      self->idlgrscene::cleanup
      Self->Remove, /All
END

FUNCTION grPanel::INIT, COLUMNS=COLUMNS, ROWS=ROWS, _EXTRA = e

    Default, ROWS, 1
    Default, COLUMNS, 1

    IF (self->IDLgrScene::Init(_EXTRA=e) NE 1) THEN RETURN, 0

    self.NRow = ROWS
    self.NCol = COLUMNS

    self.dX = 1./self.NCol
    self.dy = 1./self.NRow

    self.Row = 1 
    self.Col = 1

    return, 1
END

PRO grPanel__define

  struct = { grPanel, INHERITS IDLgrScene, tmpview:obj_new(), NCol: 0, NRow: 0, Col: 0, Row: 0, dX: 0., dY: 0. }

END
