;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; JETGET main window. 
; From this window all JETGET modules is to be accessed.
; Will read the .jetget config file in users home directory if existing.
;  If .jetget does not exists, the Setup module must be run first. 
;
;	R. Ouyed (2001)
;
; Program usage:
; > .r jetget 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    VISUALIZATION SESSION 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FUNCTION fileExists, file
; Check if the file with filename, file, exist

  ON_IOERROR, failed

  GET_LUN, lu
  OPENR,  lu, file
  FREE_LUN, lu

  RETURN, 1

  failed:
  RETURN, 0

END

PRO closeEvent, event
  Widget_Control, event.top, /DESTROY
END

PRO bmpEvent, event
;
; If the image is clicked, print the following:
;
  print, '**** JETGET (Authors: R. Ouyed, M. Jorgensen, J. Staff) ***'
  print, '****************** University of Calgary ******************'
  print, '*********** Department of Physics and Astronomy ***********'
  print, '*************** (Calgary, Alberta, Canada) ****************'
  
END

PRO output, event
  Widget_control, event.top,get_uvalue=uval
  widget_control, uval(20), set_value='ugh', /append
END 

PRO textEventMain, event
;
; What to do?
;
  widget_control,event.top,get_uvalue=textwidg
  widget_control,event.id,get_value=buttonvalue,get_uvalue=mystic
  case mystic of 
    'bmp': begin
      print, 'Welcome to JETGET'
    end
    'setup': begin
      a=obj_new('setup')
    end
    'animate': begin
      a=obj_new('animation') 
    end
    'bfield': begin
      a=obj_new('fieldlines') 
    end
    'fluxes': begin
      a=obj_new('fluxes')
    end
    'fourier': begin
      a=obj_new('fourier')
    end
    'slice': begin
      a=obj_new('slices')
    end
    'vectors': begin
      a=obj_new('vectors')
    end
    'jetset': begin
      xdisplayfile, 'readme/README-JETSET'
    end
    'quantity': begin
      a=obj_new('quantities')
    end
    'isosurf': begin
      a=obj_new('isosurface')
    end
    'readme': xdisplayfile, 'readme/README'
  endcase

END





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    WIDGET SESSION 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO jetget 
;
; This is the first routine to be run in jetget (when .r jetget is typed)
; First define a lot of common blocks.
;
  common setupvariables, basepostfix, baseprefix , n_digits, path, $
    datadir, fhdf, n_rows, n_columns, scrres, nature, angle1, $
    angle2, dim, varlist, ownvarlist, ownvarveclist, extravars, ownvecs, $
    ownveclabels, veclist, filetype, dpi, outfiletype, $
    Dimsizes, DimScaleX, DimScaleY, DimScaleZ, outdir, rdata
  COMMON new_variables, variables, varlabels, veclabels, veccont, veclabels2, $
                      veccont2, veccomp
  common time, dt
;
; Some default values
;
   default, path, "/usr/local/jetget"
   default, datadir, "/usr/local/data"
   default, baseprefix, "hdfaa000000."
   default, basepostfix, ""
   default, fhdf, 0 
   default, n_digits, 3
   default, n_rows, 4
   default, n_columns, 4
   default, dim, 3  
   default, dt, 5.0
   default, dpi, 100
   default, outfiletype, 1
   default, filetype, 0
   default, outdir, "output"
;
; Load default color table
;
  Loadct, 4, /silent
;
; Read in the .jetget file
;
  home = GETENV('HOME')
  if home EQ '' THEN $
    filename = '.jetget' $
  ELSE $
    filename = home + '/.jetget'

  if fileexists(filename) then begin
    get_lun, lun
    openr, lun, filename

    readf,lun,$
      format='(A100,A100,A100,A100,I6,I3,I2,I2,F8.3,F8.3,I3,I1,A100)',$
      basepostfix, $
      baseprefix, $
      path, $
      datadir, $
      fhdf, $
      n_digits, $
      n_rows, $
      n_columns, $
      dimm, $
      dt, $
      dpi, $
      outfiletype, $
      outdir

      dim=dimm[0]

    if dim eq 2.5 then dimsizes = intarr(3) else Dimsizes = intarr(dim)

    readf, lun, Dimsizes

    DimScaleX = fltarr(Dimsizes[0])
    DimScaleY = fltarr(Dimsizes[1])
    if (dim eq 2.5) or (dim eq 3) then begin
    DimScaleZ = fltarr(Dimsizes[2])
    endif

    readf, lun, DimScaleX
    readf, lun, DimScaleY
    if (dim eq 2.5) or (dim eq 3) then $
    readf, lun, DimScaleZ

    nv=fltarr(1)
    readf, lun, filetype
    readf, lun, nv
    nv=nv[0]
    varlist = strarr(nv)
    veclist = strarr(nv)
    readf, lun, varlist
    readf, lun, veclist
;
; calculate angles; can be used to define ones own variables in the Variable
; module.
;
    if dim eq 3 then begin
      angle1 = FLTARR(Dimsizes(0), Dimsizes(1), Dimsizes(2))
      angle2 = FLTARR(Dimsizes(0), Dimsizes(1), Dimsizes(2))
      rdata  = FLTARR(Dimsizes(1), Dimsizes(2))
      for jn=0,Dimsizes(1)-1 do begin
        for kn=0,Dimsizes(2)-1 do begin
          rdata [jn,kn] = sqrt(DimScaleY[jn]^2 + DimScaleZ[kn]^2)
          if (rdata[jn,kn] eq 0.0) then begin
            angle1[*,jn,kn] = 0.
            angle2[*,jn,kn] = 0.
          end else begin
            angle1[*,jn,kn] = DimScaleZ[kn]/rdata[jn,kn]
            angle2[*,jn,kn] = DimScaleY[jn]/rdata[jn,kn]
          end
        endfor
      endfor
    endif

;
; Figure out if the default structure is being used. If so, define the 
; extra variables.
;
    if (dim eq 2.5 and n_elements(varlist) eq 9) or (dim eq 3 and n_elements(varlist) eq 9) or (dim eq 2 and n_elements(varlist) eq 7) then begin
     if dim eq 2.5 then begin
      tosk = strarr(9)
      comp = ["v1", "v2", "v3", "b1", "b2", "b3", "d", "e1", "e2"]
      for i = 0, 8 do begin
        if varlist[i] ne comp[i] then tosk[i] = 'false'
      endfor
      if (where(tosk eq 'false'))[0] eq -1 then $
        extravars = ['LOG_d', 'LOG_e1', 'LOG_e2']
     endif
     if dim eq 3 then begin
      tosk = strarr(9)
      comp = ["v1", "v2", "v3", "b1", "b2", "b3", "d", "e1", "e2"]
      for i = 0, 8 do begin
        if varlist[i] ne comp[i] then tosk[i] = 'false'
      endfor
      if (where(tosk eq 'false'))[0] eq -1 then $
      extravars = ['v_p', 'v_phi', 'b_p', 'b_phi', 'b_phi/b_p', 'LOG_d', 'LOG_e1', 'LOG_e2', $
                          'KinEne', 'MagEne','LOG_MagEne','inclangle']
     endif
     if dim eq 2 then begin
      tosk = strarr(7)
      comp = ["v1", "v2", "b1", "b2", "d", "e1", "e2"]
      for i = 0, 6 do begin
        if varlist[i] ne comp[i] then tosk[i] = 'false'
      endfor
      if (where(tosk eq 'false'))[0] eq -1 then begin
        extravars = ['LOG_d', 'LOG_e1', 'LOG_e2']
      endif
     endif
    endif else begin
      ownvarlist = varlist
      ownvarveclist = veclist
    endelse
;
; Read the own defined variables
;
    readf, lun, n_extra
    if(n_extra) gt 0 then begin
      varlabels=strarr(n_extra)
      variables=strarr(n_extra)
      veclabels=strarr(n_extra)
      veccomp=strarr(n_extra)
      readf,lun, varlabels
      readf,lun, variables
      readf,lun, veclabels
      readf,lun, veccomp
      if n_elements(where(veclabels ne '0')) gt 2 then begin
        veccont=strarr(3,n_elements(varlabels)/3)
      endif
      veclabels2=strarr(n_elements(veclabels))
      for i=0,n_elements(veclabels)-1 do begin
        if veclabels[i] ne '0' then begin
          j=where(veclabels2 eq veclabels[i])
          if j[0] ne -1 then begin
            veccont[veccomp[i]-1, j] = variables[i]
          endif else begin
            j=where(veccont[0,*] eq '')
            veclabels2[j[0]]=veclabels[i]
            veccont[veccomp[i]-1,j[0]]=variables[i]
          endelse
        endif
      endfor 

    endif

    close,lun
    free_lun, lun
    dim=dim
  end else begin
    print, 'Configuration file does not exist.'
  end

  baseprefix=strcompress(baseprefix,/remove_all)
  basepostfix=strcompress(basepostfix,/remove_all)
  path= strcompress(path,/remove_all)
  datadir=strcompress(datadir,/remove_all)
  outdir =strcompress(outdir, /remove_all)
;
; Setup Widgets
;
  InitialBase = Widget_Base(Column=2,TITLE='JETGET - Main')

  ValueBase = WIDGET_BASE(InitialBase,COLUMN=1) ;
  OutputBase = WIDGET_BASE(InitialBase,COLUMN=1) ;

  readmebutton = widget_button(ValueBase,value='README', uvalue='readme')

  setupmain = widget_button(ValueBase,value='Setup/Configure',uvalue='setup')

   widlab = widget_label(valuebase, value='', frame=0)

  fieldmain = widget_button(ValueBase,value='Field Lines', uvalue='bfield')

  fluxesmain = widget_button(ValueBase,value='Fluxes', uvalue='fluxes')

  fouriermain = widget_button(ValueBase,value='Fourier Convolution', uvalue='fourier')

  isosurfmain = widget_button(ValueBase,value='Isosurfaces', uvalue='isosurf')

  animemain = widget_button(ValueBase,value='Mosaic',uvalue='animate')

  quantitymain = widget_button(ValueBase,value='Quantities', uvalue='quantity')

  slicemain = widget_button(ValueBase,value='Scan/Slice', uvalue='slice')

  vectorsmain = widget_button(ValueBase, value='Vectors', uvalue='vectors')

  widlab = widget_label(valuebase, value='', frame=0)

  jetsetmain = widget_button(ValueBase,value='Read about JETSET', uvalue='jetset')

  closeButton = Widget_Button(ValueBase, Value='Close', Event_Pro='closeEvent')

  geom = Widget_Info(ValueBase, /Geometry)
  YSIZE = geom.scr_ysize

  BMPButton = Widget_Button(OutputBase, ysize=ysize,xsize=ysize,event_pro='bmpevent')

  widget_control, InitialBase, set_uvalue=[readmebutton,setupmain,animemain,fieldmain,quantitymain,isosurfmain,fouriermain,slicemain,jetsetmain,closebutton]

  widget_control, OutputBase, set_uvalue=[BMPButton] 

  widget_control,BMPButton, set_value='jetget.bmp', /bitmap ; sensitive=0 
  Widget_Control, InitialBase, /REALIZE
  XManager, 'jetget', InitialBase, EVENT='textEventMain'
END
;
; compile some of the routines to use.
; 
Resolve_routine, "shared", /either, /COMPILE_FULL_FILE
Resolve_routine, "setup__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "fsc_droplist", /either, /COMPILE_FULL_FILE
Resolve_routine, "readhdf_1", /either, /COMPILE_FULL_FILE
Resolve_routine, "maxminvar_1", /either, /COMPILE_FULL_FILE
;Resolve_routine, "maxminvar", /either, /COMPILE_FULL_FILE
Resolve_routine, "grcontour__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "grpanel__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "animation__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "xpanel",  /either, /COMPILE_FULL_FILE
Resolve_routine, "fourier__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "fourierselectfilter__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "slices__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "vars__define", /either, /COMPILE_FULL_FILE
Resolve_routine, "structure__define", /either, /COMPILE_FULL_FILE
jetget
END
