;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
PRO ReadHDF_1, NID, count, SDS, FileName=FileName
;
; The routine that reads the specified variable from the HDF-file
;
; To be used in JETGET
;
  COMMON setupvariables
;
; Some default values
;
  Default, BasePrefix, 'hdfaa000000.'
  Default, BasePostfix, ''
  Default, DataDir, '.'
  Default, Count, 2
;
; An error check. Shouldn't be necessary when used in JETGET
;
  IF ((N_ELEMENTS(NID) EQ 0) AND (N_ELEMENTS(FileName) EQ 0)) THEN PRINT, 'You must specify either NID or FileName !'
;
; Generate the filename + the directory
;
  BaseName = DataDir[0]+'/'+BasePrefix[0]

  IF (N_ELEMENTS(FileName) EQ 0) THEN $
    HDFFileName = NumberFileName(BaseName,NID,BasePostFix[0], nidlength=n_digits) $
  ELSE $
    HDFFileName = DataDir[0]+'/'+FileName[0]
  fid = HDF_OPEN(HDFFileName, /READ)
;
; Sets the number of the variable to read in
;
  hdf_dfsd_readref, HDFFileName, count ; Inserted due to strange behaviour of HDF_OPEN. When opening a file, the reference number is not nescesarily reset.

  IF (fid EQ 0) THEN BEGIN
    PRINT,'readHDF_TOVS: HDFFileName.. ', HDFFileName, ' cannot be opened'
    RETURN
  ENDIF
;
; Get some info about the file
;
  DFSD_GETINFO, HDFFileName, TYPE=DataType, DIMS=Dimsizes, RANGE=range, NSDS=nsds, LASTREF=lastref, LABEL=DataLabel, UNIT=DataUnit, FORMAT=DataFormat, COORDSYS=coordsys

    rank = N_ELEMENTS(Dimsizes)

    DimLabels  = STRARR(rank)
    DimUnits   = STRARR(rank)
    DimFormats = STRARR(rank)
    dimScale  =  FLTARR(MAX(Dimsizes),rank)

    FOR i=0, rank-1 DO BEGIN
;
; Get the dimensions
;
      DFSD_DIMGET, i, LABEL=dl, UNIT=du, FORMAT=df, SCALE=ds
      DimLabels(i)  = dl
      DimUnits(i)   = du
      DimFormats(i) = df
      DimScale(0:Dimsizes(i)-1,i)   = ds
;
; Set dimscales
;
      IF (rank EQ 2) THEN BEGIN
        DimscaleX =  dimScale(0:Dimsizes(0)-1,0)
        DimscaleY =  dimScale(0:Dimsizes(1)-1,1)
        RealSDS = FLTARR(Dimsizes(0),Dimsizes(1))
      ENDIF ELSE BEGIN
        IF (rank EQ 3) THEN BEGIN
          DimscaleX = dimScale(0:Dimsizes(0)-1,0)
          DimscaleY = dimScale(0:Dimsizes(1)-1,1)
          DimscaleZ = dimScale(0:Dimsizes(2)-1,2)
          RealSDS = FLTARR(Dimsizes(0),Dimsizes(1), Dimsizes(2))
        ENDIF ELSE BEGIN
          PRINT,'readHDF: rank: ',rank,' NE 2 or 3. STOP'
          STOP
        ENDELSE
      ENDELSE
    ENDFOR
;
; Get the actual data
;
      DFSD_GETDATA, HDFFileName, SDS
      label='        '
      STRPUT,label,DataLabel 
      PRINT,DataLabel, ' read from ', HDFFileName 
      DFTAG_SDG    = 700
  
      !QUIET = 1
      HDF_DFAN_GETDESC, HDFFileName, DFTAG_SDG, lastref , tempdesc
      !QUIET = 0
       
      dataDesc = STRING(tempdesc)
  
  HDF_CLOSE, fid
  return
END
