;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; JETGET module for reading configuration input for use by JETGET.
; 
; Written by Jan Staff summer 2002
; 
; To be accessed from JETGET main widget.
;

PRO setup_event, Event

  common setupvariables
  common new_variables
  common time

  Widget_Control, event.id, Get_Uvalue=buttonvalue
  widget_control, event.top, get_uvalue=obj

  uvalue = buttonvalue
  st = obj
;
; What button was pressed?
;
  case uvalue of 

      'addbutton' : a=obj_new('vars')

      'dfsinc': a=obj_new('structure')

      'doit': obj->doit, event

      'filetype': outfiletype=event.(3)

      else   : obj->tool_event,event

    endcase
END  

PRO setup::doit, event
;
; This is the save button
;
        common setupvariables
        common new_variables
        common time
;
; Read in the values from the widget, remove the array and make them
; the right kind (not necessarily string)
;
        fpr=widget_info(event.top, find_by_uname="fnprefix")
        widget_control, fpr, get_value=baseprefix
        print, "SETUP: FilePreFix Read   = ", baseprefix
        fpo=widget_info(event.top, find_by_uname="fnpostfix")
        widget_control, fpo, get_value=basepostfix
        print, "SETUP: FilePostFix Read  = ", basepostfix
        ndig=widget_info(event.top, find_by_uname="ndigits")
        widget_control, ndig, get_value=n_digits
        print, "SETUP: FileDigits Read   = ", n_digits

        path=widget_info(event.top, find_by_uname="path")
        widget_control, path, get_value=path
        print, "SETUP: Program Path Read = ", path
        datapath=widget_info(event.top, find_by_uname="datapath")
        widget_control, datapath, get_value=datadir
        print, "SETUP: Data Path Read    = ", datadir

        n_rows=widget_info(event.top, find_by_uname="n_row")
        widget_control, n_rows, get_value=n_rows
        print, "SETUP: N_Rows Read       = ", n_rows
        n_columns=widget_info(event.top, find_by_uname="n_column")
        widget_control, n_columns, get_value=n_columns
        print, "SETUP: N_Columns Read    = ", n_columns

        ndim = widget_info(event.top, find_by_uname="dimensions")
        widget_control, ndim, get_value=dimm
        dim=float(dimm[0])
        print, "SETUP: Dimensions Read   = ", dim

        ndt = widget_info(event.top, find_by_uname="dt")
        widget_control, ndt, get_value=dt
        dt=float(dt[0])
        help, dt

        tmp=widget_info(event.top, find_by_uname="first_hdffile")
        widget_control, tmp, get_value=fhdf

        widget_control, widget_info(event.top, find_by_uname="pr_res"), get_value=dpi
        dpi=fix(dpi)
        widget_control, widget_info(event.top, find_by_uname="outfiledir"), get_value=outdir

        outdir=outdir[0]+'/'
        basepostfix=basepostfix[0]
        baseprefix=baseprefix[0]
        path=path[0]
        datadir=datadir[0]
        fhdf=fix(fhdf[0])
        n_digits=fix(n_digits[0])
        n_rows=fix(n_rows[0])
        n_columns=fix(n_columns[0])
;
; If default structure:
;
        if n_elements(ownvarlist) eq 0 then begin
;
; Set the variable list, vector list and define the extra variables, for 
; 2 or 3 dimensions
;
         if dim eq 2.5 then begin
            varlist = ['v1', 'v2', 'v3', 'b1', 'b2', 'b3', 'd', 'e1', 'e2']
            extravars = ['LOG_d', 'LOG_e1', 'LOG_e2']
            veclist = ['0','0','0','0','0','0','0','0','0']
         endif
         if dim eq 3 then begin
            varlist = ['v1', 'v2', 'v3', 'b1', 'b2', 'b3', 'd', 'e1', 'e2']
            veclist = ['v','v','v','b','b','b','0','0','0']
            extravars = ['v_p', 'v_phi', 'b_p', 'b_phi', 'b_phi/b_p', 'LOG_d', 'LOG_e1', 'LOG_e2', $
                         'KinEne', 'MagEne','LOG_MagEne','inclangle']
         endif
         if dim eq 2 then begin
            varlist = ['v1',$  
                       'v2',$  
                       'b1',$  
                       'b2',$  
                       'd', $
                       'e1',$
                       'e2']
            veclist = ['0','0','0','0','0','0','0','0']
            extravars = ['LOG_d', 'LOG_e1', 'LOG_e2']
         endif
        endif else begin
;
; Else it must be a non default structure. Use that.
;
         varlist = ownvarlist
         veclist = ownvarveclist
         if n_elements(extravars) ne 0 then temp = temporary(extravars)
        endelse
;
; Read in the first variable (to get the dimensions). It doesn't matter if
; it isn't v1
;
        if n_elements(ownvarlist) gt 0 then begin
          v1data=getdata(ownvarlist[0], fhdf) 
        endif else $ 
          v1data=getdata(varlist[0], fhdf)
;
; If the environment variable HOME is define, put the .jetget file (that
; will contain the setup information and is read at startup) in the HOME
; directory. Else put it here.
;
        home = GETENV('HOME')
        if home EQ '' THEN $ 
          filename = '.jetget' $
        ELSE $
          filename = home + '/.jetget'

        get_lun, lun

        openw, lun, filename
;
; Write the different variables to the .jetget file
;
        printf, lun, $
          format='(A100,A100,A100,A100,I6,I3,I2,I2,F8.3,F8.3,I3,I1,A100)',$
          basepostfix, $
          baseprefix, $ 
          path, $
          datadir, $
          fhdf, $   
          n_digits, $
          n_rows, $ 
          n_columns, $
          dim, $
          dt, $
          dpi, $
          outfiletype, $
          outdir

        printf, lun, Dimsizes

        printf, lun, DimScaleX
        printf, lun, DimScaleY
        if dim eq 3 then $
        printf, lun, DimScaleZ
        printf, lun, filetype
        printf, lun, n_elements(varlist)

        for i=0,n_elements(varlist)-1 do begin
         printf, lun, varlist[i]
        endfor

        for i=0,n_elements(varlist)-1 do begin
         printf, lun, veclist[i]
        endfor

        printf, lun, n_elements(varlabels)
        if n_elements(varlabels) gt 0 then begin
         for i=0,n_elements(varlabels)-1 do begin
          printf, lun, varlabels[i]
         endfor
         for i=0,n_elements(varlabels)-1 do begin
          printf, lun, variables[i]
         endfor
         for i=0,n_elements(varlabels)-1 do begin
          printf, lun, veclabels[i]
         endfor
         for i=0,n_elements(varlabels)-1 do begin
          printf, lun, veccomp[i]
         endfor
        endif  

        close, lun
        free_lun, lun
;
; calculate angles; can be used to define ones own variables in the Variable 
; module.
;
        if dim eq 3 then begin
          angle1 = FLTARR(Dimsizes(0), Dimsizes(1), Dimsizes(2))
          angle2 = FLTARR(Dimsizes(0), Dimsizes(1), Dimsizes(2))
          rdata  = FLTARR(Dimsizes(1), Dimsizes(2))
          for jn=0,Dimsizes(1)-1 do begin  
            for kn=0,Dimsizes(2)-1 do begin
              rdata [jn,kn] = sqrt(DimScaleY[jn]^2 + DimScaleZ[kn]^2)
              if (rdata[jn,kn] eq 0.0) then begin
                angle1[*,jn,kn] = 0.
                angle2[*,jn,kn] = 0.
              end else begin
                angle1[*,jn,kn] = DimScaleZ[kn]/rdata[jn,kn]
                angle2[*,jn,kn] = DimScaleY[kn]/rdata[jn,kn]
              end 
            endfor
          endfor
        endif
        print, "SETUP DONE!"
END

FUNCTION setup::init
;
; The init function for Setup
;
   common setupvariables
   common time

   default, path, "/usr/local/jetget"
   default, datadir, "/usr/local/data"
   default, baseprefix, "hdfaa000000."
   default, basepostfix, ""
   default, fhdf, 0
   default, n_digits, 3
   default, n_rows, 4
   default, n_columns, 4
   default, dim, 3
   default, dt, 10.0
   default, dpi, 100
   default, outfiletype, 1

   status= self->tool::init('Setup',/nowindow)
;
; Define the widgets
;
   path_base= widget_base(self.inbase,column=1, frame=1)

   wid_label_2=widget_label(path_base, value='Path',/align_center)
   jetpath=cw_field(path_base, uname='path', title='Path to Jetget      ',value=path)
   datapath=cw_field(path_base, uname='datapath', title='Path to data        ', value=datadir)

   hdf_base= widget_base(self.inbase,column=1, frame=1)

   wid_label_hdf=widget_label(hdf_base,value='Data-Files')

   fnprefix=cw_field(hdf_base, uname='fnprefix', title='File name prefix    ', value=baseprefix)
   fnpostfix=cw_field(hdf_base, uname='fnpostfix', title='File name postfix   ',value=basepostfix)
   ndigits=cw_field(hdf_base, uname='ndigits', title='Number of digits    ', value=n_digits)

   ndim = cw_field(hdf_base, uname="dimensions", title= "Dimensions          ",$ 
      value=dim)

   dtfield = cw_field(hdf_base, uname="dt", title='dt                  ', value=dt)

   first_hdffile=cw_field(hdf_base, uname="first_hdffile", title="First HDF file      ", value=fhdf)

   dfsinc = widget_button(hdf_base, uvalue='dfsinc', $ 
      value='Primitive Variables')

   addvarbutton = widget_button(hdf_base, uvalue='addbutton', $ 
      value='Calculate variables')
;
; output files
;
   printer_base=widget_base(self.inbase, column=1, frame=1)
   pr_label=widget_label(printer_base, value='Output files')
   filetypelist = fsc_droplist(printer_base, uvalue='filetype', uname='filetype', value=["JPEG", "PS"], index=outfiletype, title='File Type')
   pr_res=cw_field(printer_base, uname='pr_res', title='PS-file Resolution  ', value=dpi)
   outfiledirwidget=cw_field(printer_base, uname='outfiledir', uvalue='outfiledir', title='Output directory    ', value='output/')
;
; panel setup
;
   panel_base= widget_base(self.inbase,column=1, frame=1)
   wid_label_2=widget_label(panel_base, value='Panel Setup')
   n_ros=cw_field(panel_base, uname='n_row', title='Number of rows      ', value=n_rows)
   n_column=cw_field(panel_base, uname='n_column', title='Number of columns   ', value=n_columns)
;
; The buttons
;
   bbase=widget_base(self.inbase, row=1)

   doitbutton=widget_button(bbase, uvalue='doit', value='Save')
   helpbutton=widget_button(bbase, uvalue='Help', value='Help')
   closebutton=widget_button(bbase, uvalue='Close', value='Close')

   self.helpfile = "help/HELP_SETUP"
   widget_control, self.base, /update
 
   widget_control, jetpath, sensitive=0

   return, 1
END

PRO setup__define
   void = {setup, INHERITS tool}
END
