;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Jetget-tool to define a different structure for the data files.
;
; Written spring 2003 by Jan Staff.
;
; To be accessed from JETGETs Setup module.
;
; Variables:
;      ownvarlist    : the variable label
;      ownvarveclist : the label for the vector the variable is part of, or 0
;      ownvecs       : matrix containing the vector components
;      ownveclabels  : vector containing the name of the vectors in ownvecs
;
;            / c1 c2 c3 \
;  ownvecs = | h1 h2 h3 |
;            \ m1 m2 m3 /
; 
;                 / c \
;  ownveclabels = | h |
;                 \ m /
;
;  non-vector components are not included in ownvecs and ownveclabels.
;
;  ownvecs and ownveclabels has the same form as veccont and veclabels2 from
;  vars__define.
;

pro primitive_variables_event, Event
common setupvariables
;
; which button was pressed?
;
    widget_control, event.id, get_uvalue = uvalue
    widget_control, event.top, get_uvalue=obj
    if n_elements(uvalue) gt 0 then begin
      case uvalue of
       'add'     : begin
           add_row_table=widget_info(event.top, find_by_uname="structur")
           widget_control,  add_row_table, insert_rows=1, USE_TABLE_SELECT=1
       end
       'delete'  : begin
           del_row_table=widget_info(event.top, find_by_uname="structur")
           widget_control, del_row_table, delete_rows=1, USE_TABLE_SELECT=1
       end
       'default'  : obj->setdefault, event, uvalue
       'filetype' : filetype = event.(4)
       'cancel'   : obj->cancel, event
       'Close'    : obj->run, uvalue, event
       else       : obj->tool_event, event
      endcase
    endif
end

pro structure::cancel, event
;
; cancel button, just destroy the widget and exit
;
  widget_control, event.top, /destroy

end

pro structure::run, uvalue, event
;
; The save and exit button
;
  common setupvariables
;
; Read in the structure
;
  widarr=widget_info(Self.inbase, find_by_uname='structur')
  widget_control, widarr, get_value=ownvarlist
  ownvarlisttemp=transpose(ownvarlist)
  ownvarlist=ownvarlisttemp[*,0]
  ownvarveclist = ownvarlisttemp[*,1]
  if n_elements(ownveclabels) ne 0 then $
    a=temporary(ownveclabels)
;
; If no vectors at all:
;
  if where(ownvarveclist ne '0') eq [-1] then ownvarveclabels=['0']
;
; generate the ownveclabels and ownvecs, see above
;
  for loop = 0,n_elements(ownvarveclist)-1 do begin
    if ownvarveclist[loop] ne '0' then begin
      if n_elements(ownveclabels) eq 0 then begin
        ownveclabels=[ownvarveclist[loop]]
        ownvecs=[ownvarlist[loop],'0','0']
      endif else begin
        i = where(ownveclabels eq ownvarveclist[loop] )
        if i eq -1 then begin
          ownveclabels=[ownveclabels,ownvarveclist[loop]]
          ownvecs=[[ownvecs],[ownvarlist[loop],'0','0']]
        endif else begin
          j=where(ownvecs[*,i] eq '0')
          ownvecs[j[0],i]=ownvarlist[loop]
        endelse
      endelse
    endif
  endfor
  widget_control, event.top, /destroy
;
; if the structure is the default structure, set the extravars
;
  if dim eq 2.5 then begin
   tosk = strarr(9)
   comp = ["v1", "v2", "v3", "b1", "b2", "b3", "d", "e1", "e2"]
   for i = 0, 8 do begin
     if varlist[i] ne comp[i] then tosk[i] = 'false'
   endfor
   if (where(tosk eq 'false'))[0] eq -1 then $
     extravars = ['LOG_d', 'LOG_e1', 'LOG_e2']
  endif
  if (dim eq 3 and n_elements(ownvarlist) eq 9) or (dim eq 2 and n_elements(ownvarlist) eq 7) then begin
   if dim eq 3 then begin
    tosk = strarr(9)
    comp = ["v1", "v2", "v3", "b1", "b2", "b3", "d", "e1", "e2"]
    for i = 0, 8 do begin
      if ownvarlist[i] ne comp[i] then tosk[i] = 'false'
    endfor
    if (where(tosk eq 'false'))[0] eq -1 then begin
      extravars = ['v_p', 'v_phi', 'b_p', 'b_phi', 'b_phi/b_p', 'LOG_d', 'LOG_e1', 'LOG_e2', $
                        'KinEne', 'MagEne','LOG_MagEne','inclangle']
      varlist=temporary(ownvarlist)
    endif
   endif
   if dim eq 2 then begin
    tosk = strarr(7)
    comp = ["v1", "v2", "b1", "b2", "d", "e1", "e2"]
    for i = 0, 6 do begin
       if ownvarlist[i] ne comp[i] then tosk[i] = 'false'
    endfor
    if (where(tosk eq 'false'))[0] eq -1 then begin
      extravars = ['LOG_d', 'LOG_e1', 'LOG_e2']
      varlist = temporary(ownvarlist)
    endif
   endif
  endif
end

pro structure::setdefault, event, uvalue
;
; the set to default button
;
common setupvariables

  widarr=widget_info(Self.inbase, find_by_uname='structur')
  widget_control, widarr, get_value=temp
  exist=n_elements(temp[0,*])
;
; Set the default structure
;
  if dim eq 3 then begin
    val=[["v1","v"], $
         ["v2","v"], $
         ["v3","v"], $ 
         ["b1","b"], $
         ["b2","b"], $
         ["b3","b"], $ 
         ["d",'0'], $
         ["e1",'0'], $
         ["e2",'0']]
;
; Put it in the widget
;
    if exist lt 9 then $
      for i=1,9-exist do widget_control, widget_info(event.top, find_by_uname="structur"), insert_rows=1, USE_TABLE_SELECT=1
    if exist gt 9 then $
      for i=1,9-exist do widget_control, widget_info(event.top, find_by_uname="structur"), delete_rows=1, USE_TABLE_SELECT=1

; Why is there a difference on " and '? "0" won't work in the array above :-(
  endif
;
; Same for 2-d
;
  if dim eq 2 then begin
    val=[["v1",'0'], $
         ["v2",'0'], $
         ["b1",'0'], $
         ["b2",'0'], $
         ["d",'0'], $
         ["e1",'0'], $
         ["e2",'0']]

    if exist lt 8 then $
      for i=1,8-exist do widget_control, widget_info(event.top, find_by_uname="structur"), insert_rows=1, USE_TABLE_SELECT=1
    if exist gt 8 then $
      for i=1,8-exist do widget_control, widget_info(event.top, find_by_uname="structur"), delete_rows=1, USE_TABLE_SELECT=1
  endif
;
; and 2.5 d
;
  if dim eq 2.5 then begin
    val=[["v1","v"], $
         ["v2","v"], $
         ["v3","v"], $ 
         ["b1","b"], $
         ["b2","b"], $
         ["b3","b"], $ 
         ["d",'0'], $
         ["e1",'0'], $
         ["e2",'0']]
;
; Put it in the widget
;
    if exist lt 9 then $
      for i=1,9-exist do widget_control, widget_info(event.top, find_by_uname="structur"), insert_rows=1, USE_TABLE_SELECT=1
    if exist gt 9 then $
      for i=1,9-exist do widget_control, widget_info(event.top, find_by_uname="structur"), delete_rows=1, USE_TABLE_SELECT=1

; Why is there a difference on " and '? "0" won't work in the array above :-(
  endif

  widget_control, widget_info(Self.inbase, find_by_uname='structur'), $
     set_value=val

  widget_control, widget_info(self.inbase, find_by_uname="filetype"), $
     set_value=0

end

function structure::init
;
; The init function for structure
;
  common setupvariables

  status = self->Tool::Init('Primitive_Variables',/nowindow)
;
; If a self-made structure exist, use that.
;
  if n_elements(ownvarlist) gt 0 then begin
    val=strarr(2,n_elements(ownvarlist))
    val[0,*] = transpose(ownvarlist)
    val[1,*] = transpose(ownvarveclist)
  endif else begin
;
; Else use the default structure
;
    if dim eq 3 then begin
      val=[["v1","v"], $
           ["v2","v"], $
           ["v3","v"], $ 
           ["b1","b"], $
           ["b2","b"], $
           ["b3","b"], $ 
           ["d",'0'], $
           ["e1",'0'], $
           ["e2",'0']]
; Why is there a difference on " and '? "0" won't work in the array above :-(
    endif
    if dim eq 2.5 then begin
      val=[["v1","v"], $
           ["v2","v"], $
           ["v3","v"], $ 
           ["b1","b"], $
           ["b2","b"], $
           ["b3","b"], $ 
           ["d",'0'], $
           ["e1",'0'], $
           ["e2",'0']]
    endif
    if dim eq 2 then begin
      val=[["v1",'0'], $
           ["v2",'0'], $
           ["b1",'0'], $
           ["b2",'0'], $
           ["d",'0'], $
           ["e1",'0'], $
           ["e2",'0']]
    endif
  endelse
;
; The radio buttons
;
  filetypelabel=widget_label(self.inbase, /align_left, value="File Type:")
  filetype=cw_bgroup(self.inbase, ["HDF", "ASCII"], row=1, $
     /exclusive, /no_release, uname="filetype", uvalue="filetype")
;
; The table
;
  structur = widget_table(self.inbase,row_heights=60, column_widths=200,value=val $
    ,/editable,/resizeable_rows, /resizeable_columns $ 
    , column_labels=["Quantity", "Vector"]  $ 
    , uname="structur", /all_events)
;
; buttons
;
  addbutton = widget_button(self.inbase, value='Insert a variable', uname='add', uvalue='add')
  deletebutton = widget_button(self.inbase, value='Delete a variable', uname='delete', uvalue='delete')
  defaultbutton = widget_button(self.inbase, value='Set to default', uname='default', uvalue='default')
  cancelbutton = widget_button(self.inbase, value='Cancel', uname='cancel', uvalue='cancel')
  helpbutton = widget_button(self.inbase, value='Help', uname='Help', uvalue = 'Help')
  closebutton = widget_button(self.inbase, value='Save and exit', uname='Close', uvalue='Close')

  self.helpfile = 'help/HELP_STRUCTURE'

  widget_control, self.base, /update
  filetype = 0
  widget_control,widget_info(self.inbase, find_by_uname='filetype'), $
       set_value=filetype
  return, 1


end

pro structure__define
   void = {structure, INHERITS tool}
end
