;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Base object for all the tools
;
; Created Spring 2002
;   May,  2002, M. S. Joergensen
;
; Modification history:
;
;
; To be used by JETGET objects
;

PRO tool::Resize, xstretch=xstretch, units=units
;
; procedure to resize the draw window
;
common windowsize, xsize, ysize

  default, units, 2

  GEOM = WIDGET_INFO(Self.InBase,/GEOMETRY, units=units)
  YSize = GEOM.Scr_YSize
  if not(keyword_set(xstretch)) then xstretch=1.0
  if(not(keyword_set(nowindow))) then begin
    if n_elements(xstretch) ne 0 then $
     xsize=ysize*xstretch $
    else $
     xsize=ysize
    widget_control, Self.Window, YSize = YSize, XSize = xsize, units=units
  endif
  widget_control, Self.Base, /update

END

PRO tool::Tool_Event, event
;
; procedure to handle some different button-clicks from the module
;
  widget_control, event.id, get_uvalue = uvalue
  widget_control, event.top, get_uvalue=obj
  case uvalue of
    'Help'  : Self->Help
    'PS'    : Self->outfile
    'ps'    : Self->outfile
    'jpeg'  : self->outfile
    'VRML'  : Self->VRML
    'Close' : widget_control, event.top, /destroy
    'MPEG'  : obj->mpeg
    else    : print, 'unexpected uvalue=', uvalue
  end
END

PRO tool::Draw, oView, Clear=Clear, nosave=nosave
;
; Draws an image to the draw window, and also to vrml/mpeg if requested
;
  if (keyword_set(Clear)) then $
    Self.oPanel->Reset

  Self.oDev->Draw, oview
  if not(keyword_set(nosave)) then begin
    if obj_valid(self.oldview) then obj_destroy, self.oldview
    Self.oldview=oview
  endif
  if obj_valid(self.vrml) then self.vrml->draw, oview

  if obj_valid(Self.oMPEG) then begin
     Self.oImage = Self.oDev->Read()
     self.frame=self.frame+30
     for i=self.frame-30, self.frame do $
       Self.oMPEG->Put, Self.oImage, i
     obj_destroy, self.oimage
  endif

END

PRO tool::JPEG
;
; if jpeg output is requested, this routine saves the image to a jpeg file
;
common windowsize
common setupvariables

default, units, 2

   Self.oImage = Self.oDev->Read()
   self.oimage->getproperty, data=data
   self.filenameinput=obj_new('filenameinput', self, "JPEG")
   iscanceled=self.filenameinput->canceled()
   if iscanceled ne 1 then begin
    fname=self.filenameinput->getfilename()
    fname=fname[0]
    help, self.odev
     write_jpeg, fname, data, quality=75, true=1
   endif

END

PRO tool::outfile, novector=novector, units=units
;
; when the save button is pressed, the code ends up here. either jpeg or
; ps will be used.
;
  common setupvariables

  if outfiletype eq 1 then self->ps, novector=novector, units=units
  if outfiletype eq 0 then self->jpeg
end

PRO tool::PS, novector=novector, units=units
;
; if filetype is ps, this code will save the image to a postscript file.
; a postscript file can be saved either as bitmap or vector graphics.
;
common windowsize
common setupvariables

default, units, 2

   self.filenameinput=obj_new('filenameinput', self, "postscript")
   iscanceled=self.filenameinput->canceled()
   if iscanceled ne 1 then begin
    fname=self.filenameinput->getfilename()
    fname=fname[0]
    pstype=self.filenameinput->gettype()
    clipboard = Obj_New("IDLgrClipboard", dimensions=[xsize, ysize], Resolution=[2.54/dpi, 2.54/dpi], quality=2, units=units)
    if pstype eq 1 then $
     clipboard->Draw, self.oldview, Filename=fname, /PostScript $
    else $
     clipboard->Draw, self.oldview, Filename=fname, /PostScript, /vector 
    obj_destroy, clipboard
   endif
END

PRO tool::Reset_Panel, N_COLS, N_ROWS
;
; resets the panel object
;
  if (n_elements(N_COLS) EQ 0) then N_COLS = 1
  if (n_elements(N_ROWS) EQ 0) then N_ROWS = 1

  if (OBJ_VALID(Self.oPanel)) then $
    OBJ_DESTROY, Self.oPanel
  Self.oPanel = OBJ_NEW('grPanel',Columns=N_COLS,Rows=N_ROWS)
END

PRO tool::Help
  xdisplayfile, Self.HelpFile
END

PRO tool::VRML
;
; when vrml output is requested, goes here
; 
   self->VRML_init, iscanceled
   if iscanceled ne 1 then begin
    self->Run
    obj_destroy, self.vrml
   endif
END

PRO tool::VRML_init, iscanceled
;
; initiates the vrml object
;
   self.filenameinput=obj_new('filenameinput', self, "VRML")
   iscanceled=self.filenameinput->canceled()
   if iscanceled ne 1 then begin
    fname=self.filenameinput->getfilename()
    fname=fname[0]
    print, 'Generating VRML file: ', fname
    Self.vrml=obj_new('idlgrvrml', filename=fname)
   endif
END

PRO tool::MPEG
;
; if mpeg output is requested, go here
;
  Self->MPEG_init, iscanceled
  if iscanceled ne 1 then begin
   Self->Reset_Panel, 1, 1
   Self->Run
   Self->MPEG_Save
  endif
END

PRO tool::MPEG_Save, Name
;
; procedure for saving the mpeg file
;
  print, 'Saving MPEG ', Self.VarName+'.mpg'

  Self.oMPEG->Save
  obj_destroy, Self.oMPEG
  print, "Done"
END

PRO tool::MPEG_init, iscanceled
;
; initiates the mpeg object
;
   self.filenameinput=obj_new('filenameinput', self, "MPEG")
   iscanceled=self.filenameinput->canceled()
   if iscanceled ne 1 then begin
    fname=self.filenameinput->getfilename()
    fname=fname[0]

    print, 'Generating MPEG: ', fname
 
    Self.oMPEG =obj_new('IDLgrMPEG', FileName=fname)
   endif
END

PRO tool::WidInit, nowindow=nowindow, xstretch=xstretch
;
; initiates the widget for the module
;
    Self.Base = Widget_Base(Column = 2, Title = Self.Title)
    Self.InOutBase = Widget_Base(Self.Base, Column = 2)
    Self.InBase = Widget_Base(Self.InOutBase, Column = 1)
    Self.Modalbase = Widget_Base(column=1, /modal, group_leader=self.base)

    if(not(keyword_set(nowindow))) then begin
      Window = Widget_Draw(Self.InOutBase, RETAIN=2, GRAPHICS_LEVEL=2)
    end

    Self->Reset_Panel, 1, 1

    if(not(keyword_set(nowindow))) then begin
      Self.Window = Window
    end

    widget_control, Self.Base, /realize, UPDATE=0

    if(not(keyword_set(nowindow))) then begin
      widget_control, Self.Window, get_value=Window
    end

    if(not(keyword_set(nowindow))) then begin
      Self.oDev = Window
    endif

    widget_control, Self.base, set_uvalue=self                                 ; store obj in widget
    xmanager, Self.ToolType, Self.base, /no_block 

END

FUNCTION tool::INIT, Type,  xstretch=xstretch, HelpFile = HelpFile, Var=Var, NoRealize = NoRealize, nowindow = nowindow

    IF N_ELEMENTS(HelpFile) EQ 0 THEN HelpFile = ''
    IF N_ELEMENTS(Var) NE 0 THEN Self.UVar = Var

    Self.Title = 'JETGET - ' + Type 
    Self.ToolType = Type
    Self.HelpFile = HelpFile

    if not(keyword_set(NoRealize)) then $
      Self->WidInit, nowindow=nowindow

   return, 1
END

PRO tool__define

  struct = { tool, Title: '', ToolType: '', Base: 0L, ModalBase: 0L, InBase: 0L, InOutBase: 0L, HelpFile: '', VarName: '', $
             Window: 0L, oPanel: OBJ_NEW(), oDev: OBJ_NEW(), UVar: OBJ_NEW(), oMPEG: OBJ_NEW(), oImage: OBJ_NEW() $
             , oldview: obj_new(), filenameinput: obj_new(), frame: 0L $
             , vrml:obj_new()}

END
