;
; JETGET program for generating the vectors used in the Vectors module.
;
; Copyright (c) 1983-1998, Research Systems, Inc.  All rights reserved.
;       Unauthorized reproduction prohibited.
;
; Changes:
;        Spring 2003 (by Jan Staff): 
;                     Changed velovect into an object, and changed the name
;                      to ve.
;                     Allows the possibility of having both upper and 
;                       lower bounds on the vector lengths.
;                     Changed font for use with JETGETs panel.
;                     Added step argument to the function call, to allow
;                       different step between each good vector.
;

FUNCTION ve::INIT, U,V, x, y, missing1= missing1, missing2 = missing2, $ 
           usepanel=usepanel, Length= length, xmin=xmin, xmax=xmax, ymin=ymin, $
           ymax=ymax, step=step, title=title, $ 
           xtitle=xtitle, ytitle=ytitle, BTitle=BTitle

common setupvariables

  IF (Self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0

  s = size(u)
  t = size(v)
  if s[0] ne 2 then begin
     baduv:   message, 'U and V parameters must be 2D and same size.'
  endif
  if total(abs(s[0:2]-t[0:2])) ne 0 then goto,baduv
;
  if n_params(0) lt 3 then x = findgen(s[1]) else $
    if n_elements(x) ne s[1] then begin
     badxy:                  message, 'X and Y arrays have incorrect size.'
  endif
  if n_params(1) lt 4 then y = findgen(s[2]) else $
    if n_elements(y) ne s[2] then goto,badxy 
;
  if n_elements(missing1) le 0 then missing1 = 1.0e30
  if n_elements(length) le 0 then length = 1.0

  mag = sqrt(u^2.+v^2.)         ;magnitude.
                                ;Subscripts of good elements
  nbad = 0                      ;# of missing points
  if n_elements(missing1) gt 0 then begin
     good = where((mag lt missing1) and (mag gt missing2))
     if good[0] eq -1 then begin 
       print, "There are no vectors in the range you typed in. Please select another range "
       print, "The vectors are in the range:", min(mag), max(mag)
;       return,0
     endif
     if keyword_set(dots) then bad = where(mag le missing1, nbad)
  endif else begin
     good = lindgen(n_elements(mag))
  endelse

  if usepanel eq 1 then $
   ofont = obj_new('idlgrfont', 'Helvetica', size=14/n_rows) $
  else $ 
   ofont = obj_new('idlgrfont', 'Helvetica', size=14)

  oTitle = OBJ_NEW('IDLgrText', Title, font=ofont, alignment=0.5,LOCATION=[(xmax+xmin)/2,ymax+5*ymax/(Dimsizes(1)-1)])

  self.oX1Axis = OBJ_NEW('IDLgrAxis', 0)
  self.oY1Axis = OBJ_NEW('IDLgrAxis', 1)
  self.oX2Axis = OBJ_NEW('IDLgrAxis', 0)
  self.oY2Axis = OBJ_NEW('IDLgrAxis', 1)

    if (usepanel eq 1) and (n_rows gt 1) then begin
      if (float(n_rows[0]) eq 1) and (float(n_columns[0]) ne 1) then $
       ofont = obj_new('idlgrfont', 'Helvetica', size=12/2.) $
      else $
       ofont=obj_new('idlgrfont', 'Helvetica', size=12/(n_rows[0]))
      oTitlebottom = OBJ_NEW('IDLgrText', BTitle,alignment=0.5, $  
         LOCATION=[(xmax+xmin)/2,ymin-(ymax-ymin)/7.], font=ofont) 
      oXTitle = OBJ_NEW('IDLgrText', xTitle, font=ofont)
      oTitleside = obj_new('IDLgrText', yTitle, font=ofont)
      self.ox1axis->setproperty, title=oxtitle
      self.oy1axis->setproperty, title=otitleside
    endif else begin
      ofont=obj_new('idlgrfont', 'Helvetica', size=12)
      oTitlebottom = OBJ_NEW('IDLgrText', BTitle,alignment=0.5, $
         LOCATION=[(xmax+xmin)/2,ymin-(ymax-ymin)/7.], font=ofont)
      oXTitle = OBJ_NEW('IDLgrText', XTitle, font=ofont)
      oTitleside = obj_new('IDLgrText', yTitle, font=ofont)
      self.ox1axis->setproperty, title=oxtitle
      self.oy1axis->setproperty, title=otitleside
    endelse

    self.oX1Axis->SetProperty, RANGE=[Xmin, Xmax], LOCATION=[xmin, ymin], /EXACT
    self.oY1Axis->SetProperty, RANGE=[Ymin, Ymax], LOCATION=[xmin, ymin], /EXACT
    self.oX2Axis->SetProperty, RANGE=[Xmin, Xmax], LOCATION=[xmin, ymax], /EXACT, /NOTEXT, TICKDIR=1
    self.oY2Axis->SetProperty, RANGE=[Ymin, Ymax], LOCATION=[xmax,ymin], /EXACT, /NOTEXT, TICKDIR=1

;  Self.oX1Axis->SetProperty, RANGE=[Xmin, Xmax], LOCATION=[0,floor(Ymin)], /EXACT
;  Self.oY1Axis->SetProperty, RANGE=[Ymin, Ymax], LOCATION=[floor(Xmin),0], /EXACT
;  Self.oX2Axis->SetProperty, RANGE=[Xmin, Xmax], LOCATION=[0,ceil(Ymax)], /EXACT, /NOTEXT, TICKDIR=1
;  Self.oY2Axis->SetProperty, RANGE=[Ymin, Ymax], LOCATION=[ceil(Xmax),0], /EXACT, /NOTEXT, TICKDIR=1



    if (usepanel eq 1) and (n_rows gt 1) then $
      ofont=obj_new('idlgrfont', 'Helvetica', size=12/n_rows) $
    else $
      ofont=obj_new('idlgrfont', 'Helvetica', size=12)

    self.ox1axis->getproperty, ticktext=ticktext
    ticktext->setproperty, font=ofont
    self.ox2axis->getproperty, ticktext=ticktext
    ticktext->setproperty, font=ofont
    self.oy1axis->getproperty, ticktext=ticktext
    ticktext->setproperty, font=ofont
    self.oy2axis->getproperty, ticktext=ticktext
    ticktext->setproperty, font=ofont

    ticklenx=(ymax-ymin)*0.05
    tickleny=(xmax-xmin)*0.05
    self.ox1axis->setproperty, ticklen=ticklenx
    self.ox2axis->setproperty, ticklen=ticklenx
    self.oy1axis->setproperty, ticklen=tickleny
    self.oy2axis->setproperty, ticklen=tickleny


;    oTitlebottom = OBJ_NEW('IDLgrText', BTitle,alignment=0.5, $
;       LOCATION=[(xmax+xmin)/2,ymin-(ymax-ymin)/10], font=ofont)

 if good[0] ne -1 then begin
  ugood = u[good]
  vgood = v[good]
  x0 = min(x)                   ;get scaling
  x1 = max(x)
  y0 = min(y)
  y1 = max(y)
  x_step=(x1-x0)/(s[1]-1.0)     ; Convert to float. Integer math
  y_step=(y1-y0)/(s[2]-1.0)     ; could result in divide by 0   


  maxmag=max([max(abs(ugood/x_step)),max(abs(vgood/y_step))])
;  sina = length * (ugood/maxmag)
;  cosa = length * (vgood/maxmag)
;
  sina = length * ugood
  cosa = length * vgood
;
  x_b0=x0-x_step
  x_b1=x1+x_step
  y_b0=y0-y_step
  y_b1=y1+y_step


  r = .3                        ;len of arrow head
  angle = 22.5 * !dtor          ;Angle of arrowhead
  st = r * sin(angle)           ;sin 22.5 degs * length of head
  ct = r * cos(angle)

  i=0
;  for i=0L,n_elements(good)-1,step do begin ;Each point
  for j=0L,n_elements(u),step do begin ;Each point
    i=where(good eq j)
    if i ne -1 then begin
;     print, i,j
;     i=j
     x0 = x[good[i] mod s[1]]   ;get coords of start & end
     dx = sina[i]
     x1 = x0 + dx
     y0 = y[good[i] / s[1]]
     dy = cosa[i]
     y1 = y0 + dy
     xd=x_step   
     yd=y_step   
     if((xd eq 0.0) or (yd eq 0.0)) then print, "xd, yd: ", xd,yd
     Self.oPlot = $
       OBJ_NEW('IDLgrPlot',[x0,x1,x1-(ct*dx/xd-st*dy/yd)*xd, $
                            x1,x1-(ct*dx/xd+st*dy/yd)*xd], $
               [y0,y1,y1-(ct*dy/yd+st*dx/xd)*yd, $ 
                y1,y1-(ct*dy/yd-st*dx/xd)*yd])    
;      Self.oPlot->SetProperty, XCOORD_CONV=[xmin,1./float(s[1])*(xmax-xmin)] $ 
;               ,YCOORD_CONV=[ymin,1./float(s[2]-1)*(ymax-ymin)]
   
     Self->add, Self.oPlot
;     i=i+step
    endif
  endfor
 endif
  Self->Add, otitlebottom
  Self->Add, oTitle
  Self->Add, Self.oX1Axis
  Self->Add, Self.oY1Axis
  Self->Add, Self.oX2Axis
  Self->Add, Self.oY2AxiS
;  Self->Scale, 1.6, 1.6, 1.6
  return,1 
end


PRO ve__define

  struct = { ve, $
                oX1Axis: OBJ_NEW(), oX2Axis: OBJ_NEW(), oY1Axis: OBJ_NEW(), $
                oY2Axis: OBJ_NEW(), oTitle: OBJ_NEW(), oPlot: OBJ_NEW() $
            , INHERITS IDLgrModel $
           }

END
